@@ -5,7 +5,7 @@
 %%% Created : 31 Oct 2005 by Magnus Henoch <henoch@dtek.chalmers.se>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -41,19 +41,13 @@
 %% Parse an ad-hoc request.  Return either an adhoc_request record or
 %% an {error, ErrorType} tuple.
 %%
--spec(parse_request/1 ::
-(
-  IQ :: iq_request())
-    -> adhoc_response()
-    %%
-     | {error, _}
-).
+-spec parse_request(IQ :: iq_request()) -> adhoc_response() | {error, _}.
 
 parse_request(#iq{type = set, lang = Lang, sub_el = SubEl, xmlns = ?NS_COMMANDS}) ->
     ?DEBUG("entering parse_request...", []),
-    Node = xml:get_tag_attr_s(<<"node">>, SubEl),
-    SessionID = xml:get_tag_attr_s(<<"sessionid">>, SubEl),
-    Action = xml:get_tag_attr_s(<<"action">>, SubEl),
+    Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
+    SessionID = fxml:get_tag_attr_s(<<"sessionid">>, SubEl),
+    Action = fxml:get_tag_attr_s(<<"action">>, SubEl),
     XData = find_xdata_el(SubEl),
     #xmlel{children = AllEls} = SubEl,
     Others = case XData of
@@ -68,7 +62,9 @@ parse_request(#iq{type = set, lang = Lang, sub_el = SubEl, xmlns = ?NS_COMMANDS}
         xdata     = XData,
         others    = Others
     };
-parse_request(_) -> {error, ?ERR_BAD_REQUEST}.
+parse_request(#iq{lang = Lang}) ->
+    Text = <<"Failed to parse ad-hoc command request">>,
+    {error, ?ERRT_BAD_REQUEST(Lang, Text)}.
 
 %% Borrowed from mod_vcard.erl
 find_xdata_el(#xmlel{children = SubEls}) ->
@@ -76,7 +72,7 @@ find_xdata_el(#xmlel{children = SubEls}) ->
 
 find_xdata_el1([]) -> false;
 find_xdata_el1([El | Els]) when is_record(El, xmlel) ->
-    case xml:get_tag_attr_s(<<"xmlns">>, El) of
+    case fxml:get_tag_attr_s(<<"xmlns">>, El) of
         ?NS_XDATA -> El;
         _         -> find_xdata_el1(Els)
     end;
@@ -86,12 +82,9 @@ find_xdata_el1([_ | Els]) -> find_xdata_el1(Els).
 %% record, filling in values for language, node and session id from
 %% the request.
 %%
--spec(produce_response/2 ::
-(
-  Adhoc_Request  :: adhoc_request(),
-  Adhoc_Response :: adhoc_response())
-    -> Xmlel::xmlel()
-).
+-spec produce_response(Adhoc_Request :: adhoc_request(),
+		       Adhoc_Response :: adhoc_response()) ->
+			      Xmlel::xmlel().
 
 %% Produce a <command/> node to use as response from an adhoc_response
 %% record.
@@ -102,11 +95,7 @@ produce_response(#adhoc_request{lang = Lang, node = Node, sessionid = SessionID}
     }).
 
 %%
--spec(produce_response/1 ::
-(
-  Adhoc_Response::adhoc_response())
-    -> Xmlel::xmlel()
-).
+-spec produce_response(Adhoc_Response::adhoc_response()) -> Xmlel::xmlel().
 
 produce_response(
   #adhoc_response{
