@@ -5,7 +5,7 @@
 %%% Created : 7 Aug 2011 by Stephen Röttger <stephen.roettger@googlemail.com>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -159,7 +159,8 @@ mech_step(#state{step = 4} = State, ClientIn) ->
 					 ServerSignature =
 					     scram:server_signature(State#state.server_key,
 								    AuthMessage),
-					 {ok, [{username, State#state.username}],
+					 {ok, [{username, State#state.username},
+					       {authzid, State#state.username}],
 					  <<"v=",
 					    (jlib:encode_base64(ServerSignature))/binary>>};
 				     true -> {error, <<"bad-auth">>, State#state.username}
