@@ -8,7 +8,7 @@
 %%% @doc
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -28,8 +28,8 @@
 
 %%% Not implemented:
 %%% - write mod_piefxis with ejabberdctl commands
-%%% - Export from mod_offline_odbc.erl
-%%% - Export from mod_private_odbc.erl
+%%% - Export from mod_offline_sql.erl
+%%% - Export from mod_private_sql.erl
 %%% - XEP-227: 6. Security Considerations
 %%% - Other schemas of XInclude are not tested, and may not be imported correctly.
 %%% - If a host has many users, split that host in XML files with 50 users each.
@@ -64,7 +64,7 @@
 -define(NS_PIEFXIS, <<"http://www.xmpp.org/extensions/xep-0227.html#ns">>).
 -define(NS_XI, <<"http://www.w3.org/2001/XInclude">>).
 
--record(state, {xml_stream_state :: xml_stream:xml_stream_state(),
+-record(state, {xml_stream_state :: fxml_stream:xml_stream_state(),
                 user = <<"">>    :: binary(),
                 server = <<"">>  :: binary(),
                 fd               :: file:io_device(),
@@ -80,12 +80,11 @@ import_file(FileName) ->
     import_file(FileName, #state{}).
 
 -spec import_file(binary(), state()) -> ok | {error, atom()}.
-
 import_file(FileName, State) ->
     case file:open(FileName, [read, binary]) of
 	{ok, Fd} ->
             Dir = filename:dirname(FileName),
-            XMLStreamState = xml_stream:new(self(), infinity),
+            XMLStreamState = fxml_stream:new(self(), infinity),
             Res = process(State#state{xml_stream_state = XMLStreamState,
                                       fd = Fd,
                                       dir = Dir}),
@@ -97,72 +96,14 @@ import_file(FileName, State) ->
             {error, Reason}
     end.
 
-%%%==================================
-%%%% Process Elements
-%%%==================================
-%%%% Process Element
-%%%==================================
-%%%% Add user
-%% @spec (El::xmlel(), Domain::string(), User::binary(), Password::binary() | none)
-%%       -> ok | {error, ErrorText::string()}
-%% @doc Add a new user to the database.
-%% If user already exists, it will be only updated.
 -spec export_server(binary()) -> any().
-
-%% @spec (User::string(), Password::string(), Domain::string())
-%%       -> ok | {atomic, exists} | {error, not_allowed}
-%% @doc  Create a new user
 export_server(Dir) ->
     export_hosts(?MYHOSTS, Dir).
 
-%%%==================================
-%%%% Populate user
-%% @spec (User::string(), Domain::string(), El::xml())
-%%      -> ok | {error, not_found}
-%%
-%% @doc  Add a new user from a XML file with a roster list.
-%%
-%% Example of a file:
-%% ```
-%% <?xml version='1.0' encoding='UTF-8'?>
-%% <server-data xmlns='http://www.xmpp.org/extensions/xep-0227.html#ns'>
-%%   <host jid='localhost'>
-%%     <user name='juliet' password='s3crEt'>
-%%       <query xmlns='jabber:iq:roster'>
-%%         <item jid='romeo@montague.net'
-%%               name='Romeo'
-%%               subscription='both'>
-%%           <group>Friends</group>
-%%         </item>
-%%       </query>
-%%     </user>
-%%   </host>
-%%  </server-data>
-%% '''
 -spec export_host(binary(), binary()) -> any().
-
 export_host(Dir, Host) ->
     export_hosts([Host], Dir).
 
-%% @spec User   = String with the user name
-%%       Domain = String with a domain name
-%%       El     = Sub XML element with vCard tags values
-%% @ret  ok | {error, not_found}
-%% @doc  Read vcards from the XML and send it to the server
-%%
-%% Example:
-%% ```
-%% <?xml version='1.0' encoding='UTF-8'?>
-%% <server-data xmlns='http://www.xmpp.org/extensions/xep-0227.html#ns'>
-%%   <host jid='localhost'>
-%%     <user name='admin' password='s3crEt'>
-%%       <vCard xmlns='vcard-temp'>
-%%         <FN>Admin</FN>
-%%       </vCard>
-%%     </user>
-%%   </host>
-%% </server-data>
-%% '''
 %%%===================================================================
 %%% Internal functions
 %%%===================================================================
@@ -194,11 +135,6 @@ export_hosts(Hosts, Dir) ->
             {error, Reason}
     end.
 
-%% @spec User   = String with the user name
-%%       Domain = String with a domain name
-%%       El     = Sub XML element with offline messages values
-%% @ret  ok | {error, not_found}
-%% @doc  Read off-line message from the XML and send it to the server
 export_host(Dir, FnH, Host) ->
     DFn = make_host_basefilename(Dir, FnH),
     case file:open(DFn, [raw, write]) of
@@ -223,11 +159,6 @@ export_host(Dir, FnH, Host) ->
             {error, Reason}
     end.
 
-%% @spec User   = String with the user name
-%%       Domain = String with a domain name
-%%       El     = Sub XML element with private storage values
-%% @ret  ok | {error, not_found}
-%% @doc  Private storage parsing
 export_users([{User, _S}|Users], Server, Fd) ->
     case export_user(User, Server, Fd) of
         ok ->
@@ -238,8 +169,6 @@ export_users([{User, _S}|Users], Server, Fd) ->
 export_users([], _Server, _Fd) ->
     ok.
 
-%%%==================================
-%%%% Utilities
 export_user(User, Server, Fd) ->
     Password = ejabberd_auth:get_password_s(User, Server),
     LServer = jid:nameprep(Server),
@@ -257,7 +186,7 @@ export_user(User, Server, Fd) ->
         get_privacy(User, Server) ++
         get_roster(User, Server) ++
         get_private(User, Server),
-    print(Fd, xml:element_to_binary(
+    print(Fd, fxml:element_to_binary(
                 #xmlel{name = <<"user">>,
                        attrs = [{<<"name">>, User},
                                 {<<"password">>, Pass}],
@@ -289,7 +218,6 @@ get_vcard(User, Server) ->
             []
     end.
 
-%%%==================================
 get_offline(User, Server) ->
     case mod_offline:get_offline_els(User, Server) of
         [] ->
@@ -306,7 +234,6 @@ get_offline(User, Server) ->
             [#xmlel{name = <<"offline-messages">>, children = NewEls}]
     end.
 
-%%%% Export hosts
 get_privacy(User, Server) ->
     case mod_privacy:get_user_lists(User, Server) of
         {ok, #privacy{default = Default,
@@ -333,7 +260,6 @@ get_privacy(User, Server) ->
             []
     end.
 
-%% @spec (Dir::string(), Hosts::[string()]) -> ok
 get_roster(User, Server) ->
     JID = jid:make(User, Server, <<>>),
     case mod_roster:get_roster(User, Server) of
@@ -387,17 +313,17 @@ get_private(User, Server) ->
 process(#state{xml_stream_state = XMLStreamState, fd = Fd} = State) ->
     case file:read(Fd, ?CHUNK_SIZE) of
         {ok, Data} ->
-            NewXMLStreamState = xml_stream:parse(XMLStreamState, Data),
+            NewXMLStreamState = fxml_stream:parse(XMLStreamState, Data),
             case process_els(State#state{xml_stream_state =
                                              NewXMLStreamState}) of
                 {ok, NewState} ->
                     process(NewState);
                 Err ->
-                    xml_stream:close(NewXMLStreamState),
+                    fxml_stream:close(NewXMLStreamState),
                     Err
             end;
         eof ->
-            xml_stream:close(XMLStreamState),
+            fxml_stream:close(XMLStreamState),
             ok
     end.
 
@@ -415,7 +341,7 @@ process_els(State) ->
     end.
 
 process_el({xmlstreamstart, <<"server-data">>, Attrs}, State) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
         ?NS_PIEFXIS ->
             {ok, State};
         ?NS_PIE ->
@@ -430,7 +356,7 @@ process_el({xmlstreamcdata, _}, State) ->
 process_el({xmlstreamelement, #xmlel{name = <<"xi:include">>,
                                      attrs = Attrs}},
            #state{dir = Dir, user = <<"">>} = State) ->
-    FileName = xml:get_attr_s(<<"href">>, Attrs),
+    FileName = fxml:get_attr_s(<<"href">>, Attrs),
     case import_file(filename:join([Dir, FileName]), State) of
         ok ->
             {ok, State};
@@ -443,7 +369,7 @@ process_el({xmlstreamstart, <<"host">>, Attrs}, State) ->
 process_el({xmlstreamelement, #xmlel{name = <<"host">>,
                                      attrs = Attrs,
                                      children = Els}}, State) ->
-    JIDS = xml:get_attr_s(<<"jid">>, Attrs),
+    JIDS = fxml:get_attr_s(<<"jid">>, Attrs),
     case jid:from_string(JIDS) of
         #jid{lserver = S} ->
             case lists:member(S, ?MYHOSTS) of
@@ -486,8 +412,8 @@ process_users([], State) ->
 
 process_user(#xmlel{name = <<"user">>, attrs = Attrs, children = Els},
              #state{server = LServer} = State) ->
-    Name = xml:get_attr_s(<<"name">>, Attrs),
-    Password = xml:get_attr_s(<<"password">>, Attrs),
+    Name = fxml:get_attr_s(<<"name">>, Attrs),
+    Password = fxml:get_attr_s(<<"password">>, Attrs),
     PasswordFormat = ejabberd_config:get_option({auth_password_format, LServer}, fun(X) -> X end, plain),
     Pass = case PasswordFormat of
       scram ->
@@ -525,7 +451,7 @@ process_user_els([], State) ->
 
 process_user_el(#xmlel{name = Name, attrs = Attrs, children = Els} = El,
                 State) ->
-    case {Name, xml:get_attr_s(<<"xmlns">>, Attrs)} of
+    case {Name, fxml:get_attr_s(<<"xmlns">>, Attrs)} of
         {<<"query">>, ?NS_ROSTER} ->
             process_roster(El, State);
         {<<"query">>, ?NS_PRIVACY} ->
@@ -576,15 +502,13 @@ process_roster(El, State = #state{user = U, server = S}) ->
             stop("Failed to write roster: ~p", [Err])
     end.
 
-%%%==================================
-%%%% Export server
 process_privacy(El, State = #state{user = U, server = S}) ->
     JID = jid:make(U, S, <<"">>),
     case mod_privacy:process_iq_set(
            [], JID, JID, #iq{type = set, sub_el = El}) of
         {error, Error} = Err ->
             #xmlel{children = Els} = El,
-            Name = case xml:remove_cdata(Els) of
+            Name = case fxml:remove_cdata(Els) of
               [#xmlel{name = N}] -> N;
               _ -> undefined
             end,
@@ -603,7 +527,6 @@ process_privacy(El, State = #state{user = U, server = S}) ->
             {ok, State}
     end.
 
-%% @spec (Dir::string()) -> ok
 process_private(El, State = #state{user = U, server = S}) ->
     JID = jid:make(U, S, <<"">>),
     case mod_private:process_sm_iq(
@@ -614,8 +537,6 @@ process_private(El, State = #state{user = U, server = S}) ->
             stop("Failed to write private: ~p", [Err])
     end.
 
-%%%==================================
-%%%% Export host
 process_vcard(El, State = #state{user = U, server = S}) ->
     JID = jid:make(U, S, <<"">>),
     case mod_vcard:process_sm_iq(
@@ -626,9 +547,8 @@ process_vcard(El, State = #state{user = U, server = S}) ->
             stop("Failed to write vcard: ~p", [Err])
     end.
 
-%% @spec (Dir::string(), Host::string()) -> ok
 process_offline_msg(El, State = #state{user = U, server = S}) ->
-    FromS = xml:get_attr_s(<<"from">>, El#xmlel.attrs),
+    FromS = fxml:get_attr_s(<<"from">>, El#xmlel.attrs),
     case jid:from_string(FromS) of
         #jid{} = From ->
             To = jid:make(U, S, <<>>),
@@ -643,9 +563,8 @@ process_offline_msg(El, State = #state{user = U, server = S}) ->
             stop("Invalid 'from' = ~s", [FromS])
     end.
 
-%% @spec (Dir::string(), Fn::string(), Host::string()) -> ok
 process_presence(El, #state{user = U, server = S} = State) ->
-    FromS = xml:get_attr_s(<<"from">>, El#xmlel.attrs),
+    FromS = fxml:get_attr_s(<<"from">>, El#xmlel.attrs),
     case jid:from_string(FromS) of
         #jid{} = From ->
             To = jid:make(U, S, <<>>),
