@@ -5,7 +5,7 @@
 %%% Created : 15 Nov 2005 by Magnus Henoch <henoch@dtek.chalmers.se>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -35,7 +35,7 @@
 	 process_sm_iq/3, get_local_commands/5,
 	 get_local_identity/5, get_local_features/5,
 	 get_sm_commands/5, get_sm_identity/5, get_sm_features/5,
-	 ping_item/4, ping_command/4, mod_opt_type/1]).
+	 ping_item/4, ping_command/4, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -233,7 +233,7 @@ process_sm_iq(From, To, IQ) ->
     process_adhoc_request(From, To, IQ, adhoc_sm_commands).
 
 process_adhoc_request(From, To,
-		      #iq{sub_el = SubEl} = IQ, Hook) ->
+		      #iq{sub_el = SubEl, lang = Lang} = IQ, Hook) ->
     ?DEBUG("About to parse ~p...", [IQ]),
     case adhoc:parse_request(IQ) of
       {error, Error} ->
@@ -245,8 +245,9 @@ process_adhoc_request(From, To,
 	      of
 	    ignore -> ignore;
 	    empty ->
+		Txt = <<"No hook has processed this command">>,
 		IQ#iq{type = error,
-		      sub_el = [SubEl, ?ERR_ITEM_NOT_FOUND]};
+		      sub_el = [SubEl, ?ERRT_ITEM_NOT_FOUND(Lang, Txt)]};
 	    {error, Error} ->
 		IQ#iq{type = error, sub_el = [SubEl, Error]};
 	    Command -> IQ#iq{type = result, sub_el = [Command]}
@@ -277,10 +278,15 @@ ping_command(_Acc, _From, _To,
 						      [{<<"info">>,
 							translate:translate(Lang,
 									    <<"Pong">>)}]});
-       true -> {error, ?ERR_BAD_REQUEST}
+       true ->
+	    Txt = <<"Incorrect value of 'action' attribute">>,
+	    {error, ?ERRT_BAD_REQUEST(Lang, Txt)}
     end;
 ping_command(Acc, _From, _To, _Request) -> Acc.
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
 mod_opt_type(report_commands_node) ->
     fun (B) when is_boolean(B) -> B end;
