@@ -5,7 +5,7 @@
 %%% Created :  1 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 	 get_sm_identity/5, get_sm_features/5, get_sm_items/5,
 	 get_info/5, register_feature/2, unregister_feature/2,
 	 register_extra_domain/2, unregister_extra_domain/2,
-	 transform_module_options/1, mod_opt_type/1]).
+	 transform_module_options/1, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -150,9 +150,10 @@ process_local_iq_items(From, To,
 		       #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
-	  Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+	  Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 	  Host = To#jid.lserver,
 	  case ejabberd_hooks:run_fold(disco_local_items, Host,
 				       empty, [From, To, Node, Lang])
@@ -177,10 +178,11 @@ process_local_iq_info(From, To,
 		      #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  Host = To#jid.lserver,
-	  Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+	  Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 	  Identity = ejabberd_hooks:run_fold(disco_local_identity,
 					     Host, [], [From, To, Node, Lang]),
 	  Info = ejabberd_hooks:run_fold(disco_info, Host, [],
@@ -229,10 +231,12 @@ get_local_features(Acc, _From, To, <<>>, _Lang) ->
      ets:select(disco_features,
 		[{{{'_', Host}}, [], ['$_']}])
        ++ Feats};
-get_local_features(Acc, _From, _To, _Node, _Lang) ->
+get_local_features(Acc, _From, _To, _Node, Lang) ->
     case Acc of
       {result, _Features} -> Acc;
-      empty -> {error, ?ERR_ITEM_NOT_FOUND}
+      empty ->
+	    Txt = <<"No features available">>,
+	    {error, ?ERRT_ITEM_NOT_FOUND(Lang, Txt)}
     end.
 
 features_to_xml(FeatureList) ->
@@ -271,8 +275,8 @@ get_local_services(Acc, _From, To, <<>>, _Lang) ->
 get_local_services({result, _} = Acc, _From, _To, _Node,
 		   _Lang) ->
     Acc;
-get_local_services(empty, _From, _To, _Node, _Lang) ->
-    {error, ?ERR_ITEM_NOT_FOUND}.
+get_local_services(empty, _From, _To, _Node, Lang) ->
+    {error, ?ERRT_ITEM_NOT_FOUND(Lang, <<"No services available">>)}.
 
 get_vh_services(Host) ->
     Hosts = lists:sort(fun (H1, H2) ->
@@ -300,12 +304,13 @@ process_sm_iq_items(From, To,
 		    #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  case is_presence_subscribed(From, To) of
 	    true ->
 		Host = To#jid.lserver,
-		Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+		Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 		case ejabberd_hooks:run_fold(disco_sm_items, Host,
 					     empty, [From, To, Node, Lang])
 		    of
@@ -325,8 +330,9 @@ process_sm_iq_items(From, To,
 		      IQ#iq{type = error, sub_el = [SubEl, Error]}
 		end;
 	    false ->
+		Txt = <<"Not subscribed">>,
 		IQ#iq{type = error,
-		      sub_el = [SubEl, ?ERR_SERVICE_UNAVAILABLE]}
+		      sub_el = [SubEl, ?ERRT_SERVICE_UNAVAILABLE(Lang, Txt)]}
 	  end
     end.
 
@@ -347,12 +353,14 @@ get_sm_items(Acc, From,
 get_sm_items({result, _} = Acc, _From, _To, _Node,
 	     _Lang) ->
     Acc;
-get_sm_items(empty, From, To, _Node, _Lang) ->
+get_sm_items(empty, From, To, _Node, Lang) ->
     #jid{luser = LFrom, lserver = LSFrom} = From,
     #jid{luser = LTo, lserver = LSTo} = To,
     case {LFrom, LSFrom} of
       {LTo, LSTo} -> {error, ?ERR_ITEM_NOT_FOUND};
-      _ -> {error, ?ERR_NOT_ALLOWED}
+      _ ->
+	    Txt = <<"Query to another users is forbidden">>,
+	    {error, ?ERRT_NOT_ALLOWED(Lang, Txt)}
     end.
 
 is_presence_subscribed(#jid{luser = User, lserver = Server},
@@ -373,15 +381,18 @@ process_sm_iq_info(From, To,
 		   #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  case is_presence_subscribed(From, To) of
 	    true ->
 		Host = To#jid.lserver,
-		Node = xml:get_tag_attr_s(<<"node">>, SubEl),
+		Node = fxml:get_tag_attr_s(<<"node">>, SubEl),
 		Identity = ejabberd_hooks:run_fold(disco_sm_identity,
 						   Host, [],
 						   [From, To, Node, Lang]),
+		Info = ejabberd_hooks:run_fold(disco_info, Host, [],
+					       [From, To, Node, Lang]),
 		case ejabberd_hooks:run_fold(disco_sm_features, Host,
 					     empty, [From, To, Node, Lang])
 		    of
@@ -397,14 +408,15 @@ process_sm_iq_info(From, To,
 					    [{<<"xmlns">>, ?NS_DISCO_INFO}
 					     | ANode],
 					children =
-					    Identity ++
+					    Identity ++ Info ++
 					      features_to_xml(Features)}]};
 		  {error, Error} ->
 		      IQ#iq{type = error, sub_el = [SubEl, Error]}
 		end;
 	    false ->
+		Txt = <<"Not subscribed">>,
 		IQ#iq{type = error,
-		      sub_el = [SubEl, ?ERR_SERVICE_UNAVAILABLE]}
+		      sub_el = [SubEl, ?ERRT_SERVICE_UNAVAILABLE(Lang, Txt)]}
 	  end
     end.
 
@@ -421,12 +433,14 @@ get_sm_identity(Acc, _From,
 	_ -> []
       end.
 
-get_sm_features(empty, From, To, _Node, _Lang) ->
+get_sm_features(empty, From, To, _Node, Lang) ->
     #jid{luser = LFrom, lserver = LSFrom} = From,
     #jid{luser = LTo, lserver = LSTo} = To,
     case {LFrom, LSFrom} of
       {LTo, LSTo} -> {error, ?ERR_ITEM_NOT_FOUND};
-      _ -> {error, ?ERR_NOT_ALLOWED}
+      _ ->
+	    Txt = <<"Query to another users is forbidden">>,
+	    {error, ?ERRT_NOT_ALLOWED(Lang, Txt)}
     end;
 get_sm_features(Acc, _From, _To, _Node, _Lang) -> Acc.
 
@@ -520,6 +534,9 @@ values_to_xml(Values) ->
 	      end,
 	      Values).
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(extra_domains) ->
     fun (Hs) -> [iolist_to_binary(H) || H <- Hs] end;
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
