@@ -5,7 +5,7 @@
 %%% Created : 15 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -37,7 +37,7 @@
 
 -export([init/1, handle_call/3, handle_cast/2,
 	 handle_info/2, terminate/2, code_change/3,
-	 mod_opt_type/1]).
+	 mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -63,7 +63,7 @@ start_link(Host, Opts) ->
 start(Host, Opts) ->
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     ChildSpec = {Proc, {?MODULE, start_link, [Host, Opts]},
-		 temporary, 1000, worker, [?MODULE]},
+		 transient, 1000, worker, [?MODULE]},
     supervisor:start_child(ejabberd_sup, ChildSpec).
 
 stop(Host) ->
@@ -86,7 +86,7 @@ stop(Host) ->
 init([Host, Opts]) ->
     MyHost = gen_mod:get_opt_host(Host, Opts,
 				  <<"echo.@HOST@">>),
-    ejabberd_router:register_route(MyHost),
+    ejabberd_router:register_route(MyHost, Host),
     {ok, #state{host = MyHost}}.
 
 %%--------------------------------------------------------------------
@@ -118,7 +118,10 @@ handle_cast(_Msg, State) -> {noreply, State}.
 handle_info({route, From, To, Packet}, State) ->
     Packet2 = case From#jid.user of
 		<<"">> ->
-		    jlib:make_error_reply(Packet, ?ERR_BAD_REQUEST);
+		    Lang = fxml:get_tag_attr_s(<<"xml:lang">>, Packet),
+		    Txt = <<"User part of JID in 'from' is empty">>,
+		    jlib:make_error_reply(
+		      Packet, ?ERRT_BAD_REQUEST(Lang, Txt));
 		_ -> Packet
 	      end,
     do_client_version(disabled, To, From),
@@ -182,7 +185,7 @@ do_client_version(enabled, From, To) ->
     Els = receive
 	    {route, To, From2, IQ} ->
 		#xmlel{name = <<"query">>, children = List} =
-		    xml:get_subtag(IQ, <<"query">>),
+		    fxml:get_subtag(IQ, <<"query">>),
 		List
 	    after 5000 -> % Timeout in miliseconds: 5 seconds
 		      []
@@ -197,5 +200,8 @@ do_client_version(enabled, From, To) ->
     ?INFO_MSG("Information of the client: ~s~s",
 	      [ToS, Values_string2]).
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(host) -> fun iolist_to_binary/1;
 mod_opt_type(_) -> [host].
