@@ -5,7 +5,7 @@
 %%% Created : 20 Aug 2015 by Holger Weiss <holger@zedat.fu-berlin.de>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2015   ProcessOne
+%%% ejabberd, Copyright (C) 2015-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -68,6 +68,7 @@
 -export([start_link/3,
 	 start/2,
 	 stop/1,
+	 depends/2,
 	 mod_opt_type/1]).
 
 %% gen_server callbacks.
@@ -86,7 +87,8 @@
 
 %% Utility functions.
 -export([get_proc_name/2,
-	 expand_home/1]).
+	 expand_home/1,
+	 expand_host/2]).
 
 -include("ejabberd.hrl").
 -include("ejabberd_http.hrl").
@@ -108,7 +110,7 @@
 	 get_url                :: binary(),
 	 service_url            :: binary() | undefined,
 	 thumbnail              :: boolean(),
-	 slots = dict:new()     :: term()}). % dict:dict() requires Erlang 17.
+	 slots = #{}            :: map()}).
 
 -record(media_info,
 	{type   :: binary(),
@@ -177,7 +179,7 @@ mod_opt_type(host) ->
 mod_opt_type(name) ->
     fun iolist_to_binary/1;
 mod_opt_type(access) ->
-    fun(A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(max_size) ->
     fun(I) when is_integer(I), I > 0 -> I;
        (infinity) -> infinity
@@ -221,6 +223,11 @@ mod_opt_type(_) ->
      dir_mode, docroot, put_url, get_url, service_url, custom_headers,
      rm_on_unregister, thumbnail].
 
+-spec depends(binary(), gen_mod:opts()) -> [{module(), hard | soft}].
+
+depends(_Host, _Opts) ->
+    [].
+
 %%--------------------------------------------------------------------
 %% gen_server callbacks.
 %%--------------------------------------------------------------------
@@ -234,7 +241,7 @@ init({ServerHost, Opts}) ->
 			   fun iolist_to_binary/1,
 			   <<"HTTP File Upload">>),
     Access = gen_mod:get_opt(access, Opts,
-			     fun(A) when is_atom(A) -> A end,
+			     fun acl:access_rules_validator/1,
 			     local),
     MaxSize = gen_mod:get_opt(max_size, Opts,
 			      fun(I) when is_integer(I), I > 0 -> I;
@@ -273,6 +280,8 @@ init({ServerHost, Opts}) ->
     Thumbnail = gen_mod:get_opt(thumbnail, Opts,
 				fun(B) when is_boolean(B) -> B end,
 				true),
+    DocRoot1 = expand_home(str:strip(DocRoot, right, $/)),
+    DocRoot2 = expand_host(DocRoot1, ServerHost),
     case ServiceURL of
 	undefined ->
 	    ok;
@@ -289,7 +298,7 @@ init({ServerHost, Opts}) ->
 	undefined ->
 	    ok;
 	Mode ->
-	    file:change_mode(DocRoot, Mode)
+	    file:change_mode(DocRoot2, Mode)
     end,
     case Thumbnail of
 	true ->
@@ -303,13 +312,13 @@ init({ServerHost, Opts}) ->
 	false ->
 	    ok
     end,
-    ejabberd_router:register_route(Host),
+    ejabberd_router:register_route(Host, ServerHost),
     {ok, #state{server_host = ServerHost, host = Host, name = Name,
 		access = Access, max_size = MaxSize,
 		secret_length = SecretLength, jid_in_url = JIDinURL,
 		file_mode = FileMode, dir_mode = DirMode,
 		thumbnail = Thumbnail,
-		docroot = expand_home(str:strip(DocRoot, right, $/)),
+		docroot = DocRoot2,
 		put_url = expand_host(str:strip(PutURL, right, $/), ServerHost),
 		get_url = expand_host(str:strip(GetURL, right, $/), ServerHost),
 		service_url = ServiceURL}}.
@@ -318,22 +327,24 @@ init({ServerHost, Opts}) ->
       -> {reply, {ok, pos_integer(), binary(),
 		      pos_integer() | undefined,
 		      pos_integer() | undefined}, state()} |
-	 {reply, {error, binary()}, state()} | {noreply, state()}.
+	 {reply, {error, atom()}, state()} | {noreply, state()}.
 
-handle_call({use_slot, Slot}, _From, #state{file_mode = FileMode,
-					    dir_mode = DirMode,
-					    get_url = GetPrefix,
-					    thumbnail = Thumbnail,
-					    docroot = DocRoot} = State) ->
+handle_call({use_slot, Slot, Size}, _From, #state{file_mode = FileMode,
+						  dir_mode = DirMode,
+						  get_url = GetPrefix,
+						  thumbnail = Thumbnail,
+						  docroot = DocRoot} = State) ->
     case get_slot(Slot, State) of
 	{ok, {Size, Timer}} ->
 	    timer:cancel(Timer),
 	    NewState = del_slot(Slot, State),
 	    Path = str:join([DocRoot | Slot], <<$/>>),
-	    {reply, {ok, Size, Path, FileMode, DirMode, GetPrefix, Thumbnail},
+	    {reply, {ok, Path, FileMode, DirMode, GetPrefix, Thumbnail},
 	     NewState};
+	{ok, {_WrongSize, _Timer}} ->
+	    {reply, {error, size_mismatch}, State};
 	error ->
-	    {reply, {error, <<"Invalid slot">>}, State}
+	    {reply, {error, invalid_slot}, State}
     end;
 handle_call(get_docroot, _From, #state{docroot = DocRoot} = State) ->
     {reply, {ok, DocRoot}, State};
@@ -372,7 +383,7 @@ handle_info(Info, State) ->
     ?ERROR_MSG("Got unexpected info: ~p", [Info]),
     {noreply, State}.
 
--spec terminate(normal | shutdown | {shutdown, _} | _, _) -> ok.
+-spec terminate(normal | shutdown | {shutdown, _} | _, state()) -> ok.
 
 terminate(Reason, #state{server_host = ServerHost, host = Host}) ->
     ?DEBUG("Stopping HTTP upload process for ~s: ~p", [ServerHost, Reason]),
@@ -403,9 +414,8 @@ process(LocalPath, #request{method = Method, host = Host, ip = IP})
 process(_LocalPath, #request{method = 'PUT', host = Host, ip = IP,
 			     data = Data} = Request) ->
     {Proc, Slot} = parse_http_request(Request),
-    case catch gen_server:call(Proc, {use_slot, Slot}) of
-	{ok, Size, Path, FileMode, DirMode, GetPrefix, Thumbnail}
-	    when byte_size(Data) == Size ->
+    case catch gen_server:call(Proc, {use_slot, Slot, byte_size(Data)}) of
+	{ok, Path, FileMode, DirMode, GetPrefix, Thumbnail} ->
 	    ?DEBUG("Storing file from ~s for ~s: ~s",
 		   [?ADDR_TO_STR(IP), Host, Path]),
 	    case store_file(Path, Data, FileMode, DirMode,
@@ -419,13 +429,13 @@ process(_LocalPath, #request{method = 'PUT', host = Host, ip = IP,
 			       [Path, ?ADDR_TO_STR(IP), Host, ?FORMAT(Error)]),
 		    http_response(Host, 500)
 	    end;
-	{ok, Size, Path, _FileMode, _DirMode, _GetPrefix, _Thumbnail} ->
-	    ?INFO_MSG("Rejecting file ~s from ~s for ~s: Size is ~B, not ~B",
-		      [Path, ?ADDR_TO_STR(IP), Host, byte_size(Data), Size]),
+	{error, size_mismatch} ->
+	    ?INFO_MSG("Rejecting file from ~s for ~s: Unexpected size (~B)",
+		      [?ADDR_TO_STR(IP), Host, byte_size(Data)]),
 	    http_response(Host, 413);
-	{error, Error} ->
-	    ?INFO_MSG("Rejecting file from ~s for ~s: ~p",
-		      [?ADDR_TO_STR(IP), Host, Error]),
+	{error, invalid_slot} ->
+	    ?INFO_MSG("Rejecting file from ~s for ~s: Invalid slot",
+		      [?ADDR_TO_STR(IP), Host]),
 	    http_response(Host, 403);
 	Error ->
 	    ?ERROR_MSG("Cannot handle PUT request from ~s for ~s: ~p",
@@ -509,6 +519,12 @@ expand_home(Subject) ->
     Parts = binary:split(Subject, <<"@HOME@">>, [global]),
     str:join(Parts, list_to_binary(Home)).
 
+-spec expand_host(binary(), binary()) -> binary().
+
+expand_host(Subject, Host) ->
+    Parts = binary:split(Subject, <<"@HOST@">>, [global]),
+    str:join(Parts, Host).
+
 %%--------------------------------------------------------------------
 %% Internal functions.
 %%--------------------------------------------------------------------
@@ -526,7 +542,8 @@ process_iq(_From,
     IQ#iq{type = result,
 	  sub_el = [#xmlel{name = <<"query">>,
 			   attrs = [{<<"xmlns">>, ?NS_DISCO_INFO}],
-			   children = iq_disco_info(Lang, Name) ++ AddInfo}]};
+			   children = iq_disco_info(ServerHost, Lang, Name)
+					++ AddInfo}]};
 process_iq(From,
 	   #iq{type = get, xmlns = XMLNS, lang = Lang, sub_el = SubEl} = IQ,
 	   #state{server_host = ServerHost, access = Access} = State)
@@ -559,7 +576,8 @@ process_iq(From,
 	deny ->
 	    ?DEBUG("Denying HTTP upload slot request from ~s",
 		   [jid:to_string(From)]),
-	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_FORBIDDEN]}
+	    Txt = <<"Denied by ACL">>,
+	    IQ#iq{type = error, sub_el = [SubEl, ?ERRT_FORBIDDEN(Lang, Txt)]}
     end;
 process_iq(_From, #iq{sub_el = SubEl} = IQ, _State) ->
     IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
@@ -572,12 +590,12 @@ process_iq(_From, invalid, _State) ->
       -> {ok, binary(), pos_integer(), binary()} | {error, xmlel()}.
 
 parse_request(#xmlel{name = <<"request">>, attrs = Attrs} = Request, Lang) ->
-    case xml:get_attr(<<"xmlns">>, Attrs) of
+    case fxml:get_attr(<<"xmlns">>, Attrs) of
 	{value, XMLNS} when XMLNS == ?NS_HTTP_UPLOAD;
 			    XMLNS == ?NS_HTTP_UPLOAD_OLD ->
-	    case {xml:get_subtag_cdata(Request, <<"filename">>),
-		  xml:get_subtag_cdata(Request, <<"size">>),
-		  xml:get_subtag_cdata(Request, <<"content-type">>)} of
+	    case {fxml:get_subtag_cdata(Request, <<"filename">>),
+		  fxml:get_subtag_cdata(Request, <<"size">>),
+		  fxml:get_subtag_cdata(Request, <<"content-type">>)} of
 		{File, SizeStr, ContentType} when byte_size(File) > 0 ->
 		    case catch jlib:binary_to_integer(SizeStr) of
 			Size when is_integer(Size), Size > 0 ->
@@ -591,7 +609,8 @@ parse_request(#xmlel{name = <<"request">>, attrs = Attrs} = Request, Lang) ->
 		    {error, ?ERRT_BAD_REQUEST(Lang, Text)}
 	    end;
 	_ ->
-	    {error, ?ERR_BAD_REQUEST}
+	    Text = <<"No or invalid XML namespace">>,
+	    {error, ?ERRT_BAD_REQUEST(Lang, Text)}
     end;
 parse_request(_El, _Lang) -> {error, ?ERR_BAD_REQUEST}.
 
@@ -629,7 +648,7 @@ create_slot(#state{service_url = undefined,
     end;
 create_slot(#state{service_url = ServiceURL},
 	    #jid{luser = U, lserver = S} = JID, File, Size, ContentType,
-	    _Lang) ->
+	    Lang) ->
     Options = [{body_format, binary}, {full_result, false}],
     HttpOptions = [{timeout, ?SERVICE_REQUEST_TIMEOUT}],
     SizeStr = jlib:integer_to_binary(Size),
@@ -649,7 +668,8 @@ create_slot(#state{service_url = ServiceURL},
 		Lines ->
 		    ?ERROR_MSG("Can't parse data received for ~s from <~s>: ~p",
 			       [jid:to_string(JID), ServiceURL, Lines]),
-		    {error, ?ERR_SERVICE_UNAVAILABLE}
+		    Txt = <<"Failed to parse HTTP response">>,
+		    {error, ?ERRT_SERVICE_UNAVAILABLE(Lang, Txt)}
 	    end;
 	{ok, {402, _Body}} ->
 	    ?INFO_MSG("Got status code 402 for ~s from <~s>",
@@ -676,18 +696,18 @@ create_slot(#state{service_url = ServiceURL},
 -spec add_slot(slot(), pos_integer(), timer:tref(), state()) -> state().
 
 add_slot(Slot, Size, Timer, #state{slots = Slots} = State) ->
-    NewSlots = dict:store(Slot, {Size, Timer}, Slots),
+    NewSlots = maps:put(Slot, {Size, Timer}, Slots),
     State#state{slots = NewSlots}.
 
 -spec get_slot(slot(), state()) -> {ok, {pos_integer(), timer:tref()}} | error.
 
 get_slot(Slot, #state{slots = Slots}) ->
-    dict:find(Slot, Slots).
+    maps:find(Slot, Slots).
 
 -spec del_slot(slot(), state()) -> state().
 
 del_slot(Slot, #state{slots = Slots} = State) ->
-    NewSlots = dict:erase(Slot, Slots),
+    NewSlots = maps:remove(Slot, Slots),
     State#state{slots = NewSlots}.
 
 -spec slot_el(slot() | binary(), state() | binary(), binary()) -> xmlel().
@@ -737,20 +757,41 @@ map_int_to_char(N) when N =<  9 -> N + 48; % Digit.
 map_int_to_char(N) when N =< 35 -> N + 55; % Upper-case character.
 map_int_to_char(N) when N =< 61 -> N + 61. % Lower-case character.
 
--spec expand_host(binary(), binary()) -> binary().
-
-expand_host(Subject, Host) ->
-    Parts = binary:split(Subject, <<"@HOST@">>, [global]),
-    str:join(Parts, Host).
-
 -spec yield_content_type(binary()) -> binary().
 
 yield_content_type(<<"">>) -> ?DEFAULT_CONTENT_TYPE;
 yield_content_type(Type) -> Type.
 
--spec iq_disco_info(binary(), binary()) -> [xmlel()].
+-spec iq_disco_info(binary(), binary(), binary()) -> [xmlel()].
 
-iq_disco_info(Lang, Name) ->
+iq_disco_info(Host, Lang, Name) ->
+    Form = case gen_mod:get_module_opt(Host, ?MODULE, max_size,
+				       fun(I) when is_integer(I), I > 0 -> I;
+					  (infinity) -> infinity
+				       end,
+				       104857600) of
+	       infinity ->
+		   [];
+	       MaxSize ->
+		   MaxSizeStr = jlib:integer_to_binary(MaxSize),
+		   Fields = [#xmlel{name = <<"field">>,
+				    attrs = [{<<"type">>, <<"hidden">>},
+					     {<<"var">>, <<"FORM_TYPE">>}],
+				    children = [#xmlel{name = <<"value">>,
+						       children =
+							 [{xmlcdata,
+							   ?NS_HTTP_UPLOAD}]}]},
+			     #xmlel{name = <<"field">>,
+				    attrs = [{<<"var">>, <<"max-file-size">>}],
+				    children = [#xmlel{name = <<"value">>,
+						       children =
+							 [{xmlcdata,
+							   MaxSizeStr}]}]}],
+		   [#xmlel{name = <<"x">>,
+			   attrs = [{<<"xmlns">>, ?NS_XDATA},
+				    {<<"type">>, <<"result">>}],
+			   children = Fields}]
+	   end,
     [#xmlel{name = <<"identity">>,
 	    attrs = [{<<"category">>, <<"store">>},
 		     {<<"type">>, <<"file">>},
@@ -758,7 +799,7 @@ iq_disco_info(Lang, Name) ->
      #xmlel{name = <<"feature">>,
 	    attrs = [{<<"var">>, ?NS_HTTP_UPLOAD}]},
      #xmlel{name = <<"feature">>,
-	    attrs = [{<<"var">>, ?NS_HTTP_UPLOAD_OLD}]}].
+	    attrs = [{<<"var">>, ?NS_HTTP_UPLOAD_OLD}]} | Form].
 
 %% HTTP request handling.
 
@@ -796,7 +837,7 @@ store_file(Path, Data, FileMode, DirMode, GetPrefix, Slot, Thumbnail) ->
 			    {ok,
 			     [{<<"Content-Type">>,
 			       <<"text/xml; charset=utf-8">>}],
-			     xml:element_to_binary(ThumbEl)};
+			     fxml:element_to_binary(ThumbEl)};
 			pass ->
 			    ok
 		    end;
@@ -974,8 +1015,9 @@ remove_user(User, Server) ->
 					 (node) -> node
 				      end,
 				      sha1),
+    DocRoot1 = expand_host(expand_home(DocRoot), ServerHost),
     UserStr = make_user_string(jid:make(User, Server, <<"">>), JIDinURL),
-    UserDir = str:join([expand_home(DocRoot), UserStr], <<$/>>),
+    UserDir = str:join([DocRoot1, UserStr], <<$/>>),
     case del_tree(UserDir) of
 	ok ->
 	    ?INFO_MSG("Removed HTTP upload directory of ~s@~s", [User, Server]);
