@@ -7,7 +7,7 @@
 %%%              {mod_ip_blacklist, []}
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -36,7 +36,7 @@
 
 -export([update_bl_c2s/0]).
 
--export([is_ip_in_c2s_blacklist/3, mod_opt_type/1]).
+-export([is_ip_in_c2s_blacklist/3, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -65,6 +65,9 @@ preinit(Parent, State) ->
       error:_ -> Parent ! {ok, Pid, true}
     end.
 
+depends(_Host, _Opts) ->
+    [].
+
 %% TODO:
 stop(_Host) -> ok.
 
