@@ -13,6 +13,7 @@
 
 -export([start/2, stop/1, depends/2, muc_online_rooms/1,
 	 muc_unregister_nick/1, create_room/3, destroy_room/2,
+	 create_room_with_opts/4,
 	 create_rooms_file/1, destroy_rooms_file/1,
 	 rooms_unused_list/2, rooms_unused_destroy/2,
 	 get_user_rooms/2, get_room_occupants/2,
@@ -20,6 +21,7 @@
 	 change_room_option/4, get_room_options/2,
 	 set_room_affiliation/4, get_room_affiliations/2,
 	 web_menu_main/2, web_page_main/2, web_menu_host/3,
+	 subscribe_room/4, unsubscribe_room/2, get_subscribers/2,
 	 web_page_host/3, mod_opt_type/1, get_commands_spec/0]).
 
 -include("ejabberd.hrl").
@@ -87,6 +89,18 @@ get_commands_spec() ->
 		       module = ?MODULE, function = create_rooms_file,
 		       args = [{file, string}],
 		       result = {res, rescode}},
+     #ejabberd_commands{name = create_room_with_opts, tags = [muc_room],
+		       desc = "Create a MUC room name@service in host with given options",
+		       module = ?MODULE, function = create_room_with_opts,
+		       args = [{name, binary}, {service, binary},
+			       {host, binary},
+			       {options, {list,
+					  {option, {tuple,
+						    [{name, binary},
+						     {value, binary}
+						    ]}}
+					 }}],
+		       result = {res, rescode}},
      #ejabberd_commands{name = destroy_rooms_file, tags = [muc],
 		       desc = "Destroy the rooms indicated in file",
 		       longdesc = "Provide one room JID per line.",
@@ -151,7 +165,22 @@ get_commands_spec() ->
 								 {value, string}
 								]}}
 						}}},
-
+     #ejabberd_commands{name = subscribe_room, tags = [muc_room],
+			desc = "Subscribe to a MUC conference",
+			module = ?MODULE, function = subscribe_room,
+			args = [{user, binary}, {nick, binary}, {room, binary},
+				{nodes, binary}],
+			result = {nodes, {list, {node, string}}}},
+     #ejabberd_commands{name = unsubscribe_room, tags = [muc_room],
+			desc = "Unsubscribe from a MUC conference",
+			module = ?MODULE, function = unsubscribe_room,
+			args = [{user, binary}, {room, binary}],
+			result = {res, rescode}},
+     #ejabberd_commands{name = get_subscribers, tags = [muc_room],
+			desc = "List subscribers of a MUC conference",
+			module = ?MODULE, function = get_subscribers,
+			args = [{name, binary}, {service, binary}],
+			result = {subscribers, {list, {jid, string}}}},
      #ejabberd_commands{name = set_room_affiliation, tags = [muc_room],
 		       desc = "Change an affiliation in a MUC room",
 		       module = ?MODULE, function = set_room_affiliation,
@@ -400,15 +429,23 @@ prepare_room_info(Room_info) ->
 %%       ok | error
 %% @doc Create a room immediately with the default options.
 create_room(Name1, Host1, ServerHost) ->
+    create_room_with_opts(Name1, Host1, ServerHost, []).
+
+create_room_with_opts(Name1, Host1, ServerHost, CustomRoomOpts) ->
     Name = jid:nodeprep(Name1),
     Host = jid:nodeprep(Host1),
 
     %% Get the default room options from the muc configuration
     DefRoomOpts = gen_mod:get_module_opt(ServerHost, mod_muc,
 					 default_room_options, fun(X) -> X end, []),
+    %% Change default room options as required
+    FormattedRoomOpts = [format_room_option(Opt, Val) || {Opt, Val}<-CustomRoomOpts],
+    RoomOpts = lists:ukeymerge(1,
+                               lists:keysort(1, FormattedRoomOpts),
+                               lists:keysort(1, DefRoomOpts)),
 
     %% Store the room on the server, it is not started yet though at this point
-    mod_muc:store_room(ServerHost, Host, Name, DefRoomOpts),
+    mod_muc:store_room(ServerHost, Host, Name, RoomOpts),
 
     %% Get all remaining mod_muc parameters that might be utilized
     Access = gen_mod:get_module_opt(ServerHost, mod_muc, access, fun(X) -> X end, all),
@@ -429,7 +466,7 @@ create_room(Name1, Host1, ServerHost) ->
 			  Name,
 			  HistorySize,
 			  RoomShaper,
-			  DefRoomOpts),
+			  RoomOpts),
 	    {atomic, ok} = register_room(Host, Name, Pid),
 	    ok;
 	_ ->
@@ -748,12 +785,20 @@ send_direct_invitation(FromJid, UserJid, XmlEl) ->
 %% the option to change (for example title or max_users),
 %% and the value to assign to the new option.
 %% For example:
-%%   change_room_option("testroom", "conference.localhost", "title", "Test Room")
-change_room_option(Name, Service, Option, Value) when is_atom(Option) ->
-    Pid = get_room_pid(Name, Service),
-    {ok, _} = change_room_option(Pid, Option, Value),
-    ok;
+%%   change_room_option(<<"testroom">>, <<"conference.localhost">>, <<"title">>, <<"Test Room">>)
 change_room_option(Name, Service, OptionString, ValueString) ->
+    case get_room_pid(Name, Service) of
+	room_not_found ->
+	    room_not_found;
+	Pid ->
+	    {Option, Value} = format_room_option(OptionString, ValueString),
+	    Config = get_room_config(Pid),
+	    Config2 = change_option(Option, Value, Config),
+	    {ok, _} = gen_fsm:sync_send_all_state_event(Pid, {change_config, Config2}),
+	    ok
+    end.
+
+format_room_option(OptionString, ValueString) ->
     Option = jlib:binary_to_atom(OptionString),
     Value = case Option of
 		title -> ValueString;
@@ -764,12 +809,7 @@ change_room_option(Name, Service, OptionString, ValueString) ->
 		max_users -> jlib:binary_to_integer(ValueString);
 		_ -> jlib:binary_to_atom(ValueString)
 	    end,
-    change_room_option(Name, Service, Option, Value).
-
-change_room_option(Pid, Option, Value) ->
-    Config = get_room_config(Pid),
-    Config2 = change_option(Option, Value, Config),
-    gen_fsm:sync_send_all_state_event(Pid, {change_config, Config2}).
+    {Option, Value}.
 
 %% @doc Get the Pid of an existing MUC room, or 'room_not_found'.
 get_room_pid(Name, Service) ->
@@ -789,6 +829,7 @@ change_option(Option, Value, Config) ->
 	allow_private_messages -> Config#config{allow_private_messages = Value};
 	allow_private_messages_from_visitors -> Config#config{allow_private_messages_from_visitors = Value};
 	allow_query_users -> Config#config{allow_query_users = Value};
+	allow_subscription -> Config#config{allow_subscription = Value};
 	allow_user_invites -> Config#config{allow_user_invites = Value};
 	allow_visitor_nickchange -> Config#config{allow_visitor_nickchange = Value};
 	allow_visitor_status -> Config#config{allow_visitor_status = Value};
@@ -884,6 +925,74 @@ set_room_affiliation(Name, Service, JID, AffiliationString) ->
 	    error
     end.
 
+%%%
+%%% MUC Subscription
+%%%
+
+subscribe_room(_User, Nick, _Room, _Nodes) when Nick == <<"">> ->
+    throw({error, "Nickname must be set"});
+subscribe_room(User, Nick, Room, Nodes) ->
+    NodeList = re:split(Nodes, "\\h*,\\h*"),
+    case jid:from_string(Room) of
+	#jid{luser = Name, lserver = Host} when Name /= <<"">> ->
+	    case jid:from_string(User) of
+		error ->
+		    throw({error, "Malformed user JID"});
+		#jid{lresource = <<"">>} ->
+		    throw({error, "User's JID should have a resource"});
+		UserJID ->
+		    case get_room_pid(Name, Host) of
+			Pid when is_pid(Pid) ->
+			    case gen_fsm:sync_send_all_state_event(
+				   Pid,
+				   {muc_subscribe, UserJID, Nick, NodeList}) of
+				{ok, SubscribedNodes} ->
+				    SubscribedNodes;
+				{error, Reason} ->
+				    throw({error, binary_to_list(Reason)})
+			    end;
+			_ ->
+			    throw({error, "The room does not exist"})
+		    end
+	    end;
+	_ ->
+	    throw({error, "Malformed room JID"})
+    end.
+
+unsubscribe_room(User, Room) ->
+    case jid:from_string(Room) of
+	#jid{luser = Name, lserver = Host} when Name /= <<"">> ->
+	    case jid:from_string(User) of
+		error ->
+		    throw({error, "Malformed user JID"});
+		UserJID ->
+		    case get_room_pid(Name, Host) of
+			Pid when is_pid(Pid) ->
+			    case gen_fsm:sync_send_all_state_event(
+				   Pid,
+				   {muc_unsubscribe, UserJID}) of
+				ok ->
+				    ok;
+				{error, Reason} ->
+				    throw({error, binary_to_list(Reason)})
+			    end;
+			_ ->
+			    throw({error, "The room does not exist"})
+		    end
+	    end;
+	_ ->
+	    throw({error, "Malformed room JID"})
+    end.
+
+get_subscribers(Name, Host) ->
+    case get_room_pid(Name, Host) of
+	Pid when is_pid(Pid) ->
+	    {ok, JIDList} = gen_fsm:sync_send_all_state_event(Pid, get_subscribers),
+	    [jid:to_string(jid:remove_resource(J)) || J <- JIDList];
+	_ ->
+	    throw({error, "The room does not exist"})
+    end.
+
 make_opts(StateData) ->
     Config = StateData#state.config,
     [
