@@ -5,7 +5,7 @@
 %%% Created : 29 May 2007 by Badlop <badlop@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 -export([init/1, handle_info/2, handle_call/3,
 	 handle_cast/2, terminate/2, code_change/3]).
 
--export([purge_loop/1, mod_opt_type/1]).
+-export([purge_loop/1, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -140,7 +140,7 @@ init([LServerS, Opts]) ->
     LServiceS = gen_mod:get_opt_host(LServerS, Opts,
 				     <<"multicast.@HOST@">>),
     Access = gen_mod:get_opt(access, Opts,
-			     fun (A) when is_atom(A) -> A end, all),
+			     fun acl:access_rules_validator/1, all),
     SLimits =
 	build_service_limit_record(gen_mod:get_opt(limits, Opts,
 						   fun (A) when is_list(A) ->
@@ -151,7 +151,7 @@ init([LServerS, Opts]) ->
     try_start_loop(),
     create_pool(),
     ejabberd_router_multicast:register_route(LServerS),
-    ejabberd_router:register_route(LServiceS),
+    ejabberd_router:register_route(LServiceS, LServerS),
     {ok,
      #state{lservice = LServiceS, lserver = LServerS,
 	    access = Access, service_limits = SLimits}}.
@@ -233,7 +233,7 @@ handle_iq(From, To, #xmlel{attrs = Attrs} = Packet, State) ->
             ejabberd_router:route(To, From, Err);
         reply ->
             LServiceS = jts(To),
-            case xml:get_attr_s(<<"type">>, Attrs) of
+            case fxml:get_attr_s(<<"type">>, Attrs) of
                 <<"result">> ->
                     process_iqreply_result(From, LServiceS, Packet, State);
                 <<"error">> ->
@@ -304,7 +304,7 @@ iq_vcard(Lang) ->
 		[{xmlcdata,
                   <<(translate:translate(Lang,
                                       <<"ejabberd Multicast service">>))/binary,
-                                        "\nCopyright (c) 2002-2015 ProcessOne">>}]}].
+                                        "\nCopyright (c) 2002-2016 ProcessOne">>}]}].
 
 %%%-------------------------
 %%% Route
@@ -395,7 +395,7 @@ act_groups(FromJID, Packet_stripped, AAttrs, LServiceS,
 perform(From, Packet, AAttrs, _,
 	{route_single, Group}) ->
     [route_packet(From, ToUser, Packet, AAttrs,
-		  Group#group.addresses)
+		  Group#group.others, Group#group.addresses)
      || ToUser <- Group#group.dests];
 perform(From, Packet, AAttrs, _,
 	{{route_multicast, JID, RLimits}, Group}) ->
@@ -436,17 +436,17 @@ check_access(LServerS, Access, From) ->
 %%%-------------------------
 
 strip_addresses_element(Packet) ->
-    case xml:get_subtag(Packet, <<"addresses">>) of
+    case fxml:get_subtag(Packet, <<"addresses">>) of
       #xmlel{name = <<"addresses">>, attrs = AAttrs,
 	     children = Addresses} ->
-	  case xml:get_attr_s(<<"xmlns">>, AAttrs) of
+	  case fxml:get_attr_s(<<"xmlns">>, AAttrs) of
 	    ?NS_ADDRESS ->
 		#xmlel{name = Name, attrs = Attrs, children = Els} =
 		    Packet,
 		Els_stripped = lists:keydelete(<<"addresses">>, 2, Els),
 		Packet_stripped = #xmlel{name = Name, attrs = Attrs,
 					 children = Els_stripped},
-		{ok, Packet_stripped, AAttrs, xml:remove_cdata(Addresses)};
+		{ok, Packet_stripped, AAttrs, fxml:remove_cdata(Addresses)};
 	    _ -> throw(ewxmlns)
 	  end;
       _ -> throw(eadsele)
@@ -460,10 +460,10 @@ split_addresses_todeliver(Addresses) ->
     lists:partition(fun (XML) ->
 			    case XML of
 			      #xmlel{name = <<"address">>, attrs = Attrs} ->
-				  case xml:get_attr_s(<<"delivered">>, Attrs) of
+				  case fxml:get_attr_s(<<"delivered">>, Attrs) of
 				    <<"true">> -> false;
 				    _ ->
-					Type = xml:get_attr_s(<<"type">>,
+					Type = fxml:get_attr_s(<<"type">>,
 							      Attrs),
 					case Type of
 					  <<"to">> -> true;
@@ -499,10 +499,10 @@ check_limit_dests(SLimits, FromJID, Packet,
 
 convert_dest_record(XMLs) ->
     lists:map(fun (XML) ->
-		      case xml:get_tag_attr_s(<<"jid">>, XML) of
+		      case fxml:get_tag_attr_s(<<"jid">>, XML) of
 			<<"">> -> #dest{jid_string = none, full_xml = XML};
 			JIDS ->
-			    Type = xml:get_tag_attr_s(<<"type">>, XML),
+			    Type = fxml:get_tag_attr_s(<<"type">>, XML),
 			    JIDJ = stj(JIDS),
 			    #dest{jid_string = JIDS, jid_jid = JIDJ,
 				  type = Type, full_xml = XML}
@@ -525,7 +525,7 @@ split_dests_jid(Dests) ->
 		    Dests).
 
 report_not_jid(From, Packet, Dests) ->
-    Dests2 = [xml:element_to_binary(Dest#dest.full_xml)
+    Dests2 = [fxml:element_to_binary(Dest#dest.full_xml)
 	      || Dest <- Dests],
     [route_error(From, From, Packet, jid_malformed,
 		 <<"This service can not process the address: ",
@@ -634,13 +634,13 @@ decide_action_group(Group) ->
 %%% Route packet
 %%%-------------------------
 
-route_packet(From, ToDest, Packet, AAttrs, Addresses) ->
+route_packet(From, ToDest, Packet, AAttrs, Others, Addresses) ->
     Dests = case ToDest#dest.type of
 	      <<"bcc">> -> [];
 	      _ -> [ToDest]
 	    end,
     route_packet2(From, ToDest#dest.jid_string, Dests,
-		  Packet, AAttrs, Addresses).
+		  Packet, AAttrs, {Others, Addresses}).
 
 route_packet_multicast(From, ToS, Packet, AAttrs, Dests,
 		       Addresses, Limits) ->
@@ -666,6 +666,8 @@ route_packet2(From, ToS, Dests, Packet, AAttrs,
     ToJID = stj(ToS),
     ejabberd_router:route(From, ToJID, Packet2).
 
+append_dests(_Dests, {Others, Addresses}) ->
+    Addresses++Others;
 append_dests([], Addresses) -> Addresses;
 append_dests([Dest | Dests], Addresses) ->
     append_dests(Dests, [Dest#dest.full_xml | Addresses]).
@@ -734,8 +736,8 @@ process_iqreply_error(From, LServiceS, _Packet) ->
 process_iqreply_result(From, LServiceS, Packet, State) ->
     #xmlel{name = <<"query">>, attrs = Attrs2,
 	   children = Els2} =
-	xml:get_subtag(Packet, <<"query">>),
-    case xml:get_attr_s(<<"xmlns">>, Attrs2) of
+	fxml:get_subtag(Packet, <<"query">>),
+    case fxml:get_attr_s(<<"xmlns">>, Attrs2) of
       ?NS_DISCO_INFO ->
 	  process_discoinfo_result(From, LServiceS, Els2, State);
       ?NS_DISCO_ITEMS ->
@@ -763,7 +765,7 @@ process_discoinfo_result2(From, FromS, LServiceS, Els,
 	    fun(XML) ->
 		    case XML of
 			#xmlel{name = <<"feature">>, attrs = Attrs} ->
-			    (?NS_ADDRESS) == xml:get_attr_s(<<"var">>, Attrs);
+			    (?NS_ADDRESS) == fxml:get_attr_s(<<"var">>, Attrs);
 			_ -> false
 		    end
 	    end,
@@ -807,10 +809,10 @@ get_limits_els(Els) ->
 			  #xmlel{name = <<"x">>, attrs = Attrs,
 				 children = SubEls} ->
 			      case ((?NS_XDATA) ==
-				      xml:get_attr_s(<<"xmlns">>, Attrs))
+				      fxml:get_attr_s(<<"xmlns">>, Attrs))
 				     and
 				     (<<"result">> ==
-					xml:get_attr_s(<<"type">>, Attrs))
+					fxml:get_attr_s(<<"type">>, Attrs))
 				  of
 				true -> get_limits_fields(SubEls) ++ R;
 				false -> R
@@ -826,11 +828,11 @@ get_limits_fields(Fields) ->
 					     #xmlel{name = <<"field">>,
 						    attrs = Attrs} ->
 						 (<<"FORM_TYPE">> ==
-						    xml:get_attr_s(<<"var">>,
+						    fxml:get_attr_s(<<"var">>,
 								   Attrs))
 						   and
 						   (<<"hidden">> ==
-						      xml:get_attr_s(<<"type">>,
+						      fxml:get_attr_s(<<"type">>,
 								     Attrs));
 					     _ -> false
 					   end
@@ -848,8 +850,8 @@ get_limits_values(Values) ->
 				 children = SubEls} ->
 			      [#xmlel{name = <<"value">>, children = SubElsV}] =
 				  SubEls,
-			      Number = xml:get_cdata(SubElsV),
-			      Name = xml:get_attr_s(<<"var">>, Attrs),
+			      Number = fxml:get_cdata(SubElsV),
+			      Name = fxml:get_attr_s(<<"var">>, Attrs),
 			      [{jlib:binary_to_atom(Name),
 				jlib:binary_to_integer(Number)}
 			       | R];
@@ -870,7 +872,7 @@ process_discoitems_result(From, LServiceS, Els) ->
                      fun(XML, Res) ->
                              case XML of
                                  #xmlel{name = <<"item">>, attrs = Attrs} ->
-                                     SJID = xml:get_attr_s(<<"jid">>, Attrs),
+                                     SJID = fxml:get_attr_s(<<"jid">>, Attrs),
                                      case jid:from_string(SJID) of
                                          #jid{luser = <<"">>,
                                               lresource = <<"">>} ->
@@ -912,8 +914,9 @@ received_awaiter(JID, Waiter, LServiceS) ->
 		From = Waiter#waiter.sender,
 		Packet = Waiter#waiter.packet,
 		AAttrs = Waiter#waiter.aattrs,
+		Others = Group#group.others,
 		Addresses = Waiter#waiter.addresses,
-		[route_packet(From, ToUser, Packet, AAttrs, Addresses)
+		[route_packet(From, ToUser, Packet, AAttrs, Others, Addresses)
 		 || ToUser <- Group#group.dests];
 	    true ->
 		send_query_info(RServer, LServiceS),
@@ -1196,7 +1199,7 @@ to_binary(A) -> list_to_binary(hd(io_lib:format("~p", [A]))).
 
 route_error(From, To, Packet, ErrType, ErrText) ->
     #xmlel{attrs = Attrs} = Packet,
-    Lang = xml:get_attr_s(<<"xml:lang">>, Attrs),
+    Lang = fxml:get_attr_s(<<"xml:lang">>, Attrs),
     Reply = make_reply(ErrType, Lang, ErrText),
     Err = jlib:make_error_reply(Packet, Reply),
     ejabberd_router:route(From, To, Err).
@@ -1216,8 +1219,11 @@ stj(String) -> jid:from_string(String).
 
 jts(String) -> jid:to_string(String).
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(access) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(host) -> fun iolist_to_binary/1;
 mod_opt_type(limits) ->
     fun (A) when is_list(A) -> A end;
