@@ -5,7 +5,7 @@
 %%% Created : 12 Oct 2006 by Evgeniy Khramtsov <xram@jabber.ru>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 %% supervisor callbacks.
 -export([init/1]).
 
--export([start_link/2, mod_opt_type/1]).
+-export([start_link/2, mod_opt_type/1, depends/2]).
 
 -define(PROCNAME, ejabberd_mod_proxy65).
 
@@ -84,6 +84,9 @@ init([Host, Opts]) ->
      {{one_for_one, 10, 1},
       [StreamManager, StreamSupervisor, Service]}}.
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(auth_type) ->
     fun (plain) -> plain;
 	(anonymous) -> anonymous
