@@ -5,7 +5,7 @@
 %%% Created : 12 Oct 2006 by Evgeniy Khramtsov <xram@jabber.ru>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -63,7 +63,7 @@ start_link(Host, Opts) ->
 
 init([Host, Opts]) ->
     State = parse_options(Host, Opts),
-    ejabberd_router:register_route(State#state.myhost),
+    ejabberd_router:register_route(State#state.myhost, Host),
     {ok, State}.
 
 terminate(_Reason, #state{myhost = MyHost}) ->
@@ -150,7 +150,7 @@ process_iq(_,
 		      children = iq_vcard(Lang)}]};
 %% bytestreams info request
 process_iq(JID,
-	   #iq{type = get, sub_el = SubEl,
+	   #iq{type = get, sub_el = SubEl, lang = Lang,
 	       xmlns = ?NS_BYTESTREAMS} =
 	       IQ,
 	   #state{acl = ACL, stream_addr = StreamAddr,
@@ -165,21 +165,22 @@ process_iq(JID,
 			    attrs = [{<<"xmlns">>, ?NS_BYTESTREAMS}],
 			    children = StreamHostEl}]};
       deny ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_FORBIDDEN]}
+	  Txt = <<"Denied by ACL">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_FORBIDDEN(Lang, Txt)]}
     end;
 %% bytestream activation request
 process_iq(InitiatorJID,
-	   #iq{type = set, sub_el = SubEl,
+	   #iq{type = set, sub_el = SubEl, lang = Lang,
 	       xmlns = ?NS_BYTESTREAMS} =
 	       IQ,
 	   #state{acl = ACL, serverhost = ServerHost}) ->
     case acl:match_rule(ServerHost, ACL, InitiatorJID) of
       allow ->
-	  ActivateEl = xml:get_path_s(SubEl,
+	  ActivateEl = fxml:get_path_s(SubEl,
 				      [{elem, <<"activate">>}]),
-	  SID = xml:get_tag_attr_s(<<"sid">>, SubEl),
+	  SID = fxml:get_tag_attr_s(<<"sid">>, SubEl),
 	  case catch
-		 jid:from_string(xml:get_tag_cdata(ActivateEl))
+		 jid:from_string(fxml:get_tag_cdata(ActivateEl))
 	      of
 	    TargetJID
 		when is_record(TargetJID, jid), SID /= <<"">>,
@@ -194,22 +195,27 @@ process_iq(InitiatorJID,
 		    of
 		  ok -> IQ#iq{type = result, sub_el = []};
 		  false ->
+		      Txt = <<"Failed to activate bytestream">>,
 		      IQ#iq{type = error,
-			    sub_el = [SubEl, ?ERR_ITEM_NOT_FOUND]};
+			    sub_el = [SubEl, ?ERRT_ITEM_NOT_FOUND(Lang, Txt)]};
 		  limit ->
+		      Txt = <<"Too many active bytestreams">>,
 		      IQ#iq{type = error,
-			    sub_el = [SubEl, ?ERR_RESOURCE_CONSTRAINT]};
+			    sub_el = [SubEl, ?ERRT_RESOURCE_CONSTRAINT(Lang, Txt)]};
 		  conflict ->
-		      IQ#iq{type = error, sub_el = [SubEl, ?ERR_CONFLICT]};
+		      Txt = <<"Bytestream already activated">>,
+		      IQ#iq{type = error, sub_el = [SubEl, ?ERRT_CONFLICT(Lang, Txt)]};
 		  _ ->
 		      IQ#iq{type = error,
 			    sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
 		end;
 	    _ ->
-		IQ#iq{type = error, sub_el = [SubEl, ?ERR_BAD_REQUEST]}
+		Txt = <<"Malformed JID">>,
+		IQ#iq{type = error, sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt)]}
 	  end;
       deny ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_FORBIDDEN]}
+	  Txt = <<"Denied by ACL">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_FORBIDDEN(Lang, Txt)]}
     end;
 %% Unknown "set" or "get" request
 process_iq(_, #iq{type = Type, sub_el = SubEl} = IQ, _)
@@ -246,7 +252,7 @@ iq_vcard(Lang) ->
 		[{xmlcdata,
 		  <<(translate:translate(Lang,
 					 <<"ejabberd SOCKS5 Bytestreams module">>))/binary,
-		    "\nCopyright (c) 2003-2015 ProcessOne">>}]}].
+		    "\nCopyright (c) 2003-2016 ProcessOne">>}]}].
 
 parse_options(ServerHost, Opts) ->
     MyHost = gen_mod:get_opt_host(ServerHost, Opts,
@@ -254,7 +260,7 @@ parse_options(ServerHost, Opts) ->
     Port = gen_mod:get_opt(port, Opts,
                            fun(P) when is_integer(P), P>0, P<65536 -> P end,
                            7777),
-    ACL = gen_mod:get_opt(access, Opts, fun(A) when is_atom(A) -> A end,
+    ACL = gen_mod:get_opt(access, Opts, fun acl:access_rules_validator/1,
                           all),
     Name = gen_mod:get_opt(name, Opts, fun iolist_to_binary/1,
 			   <<"SOCKS5 Bytestreams">>),
