@@ -4,7 +4,7 @@
 %%% Purpose : Bytestream process.
 %%% Created : 12 Oct 2006 by Evgeniy Khramtsov <xram@jabber.ru>
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -83,7 +83,7 @@ init([Socket, Host, Opts]) ->
                                   (anonymous) -> anonymous
                                end, anonymous),
     Shaper = gen_mod:get_opt(shaper, Opts,
-                             fun(A) when is_atom(A) -> A end,
+                             fun acl:shaper_rules_validator/1,
                              none),
     RecvBuf = gen_mod:get_opt(recbuf, Opts,
                               fun(I) when is_integer(I), I>0 -> I end,
@@ -153,7 +153,7 @@ wait_for_auth(Packet,
 	      #state{socket = Socket, host = Host} = StateData) ->
     case mod_proxy65_lib:unpack_auth_request(Packet) of
       {User, Pass} ->
-	  Result = ejabberd_auth:check_password(User, Host, Pass),
+	  Result = ejabberd_auth:check_password(User, <<"">>, Host, Pass),
 	  gen_tcp:send(Socket,
 		       mod_proxy65_lib:make_auth_reply(Result)),
 	  case Result of
