@@ -5,7 +5,7 @@
 %%% Created :  8 Dec 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -37,7 +37,7 @@
 	 unauthenticated_iq_register/4, try_register/5,
 	 process_iq/3, send_registration_notifications/3,
 	 transform_options/1, transform_module_options/1,
-	 mod_opt_type/1, opt_type/1]).
+	 mod_opt_type/1, opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -72,9 +72,12 @@ stop(Host) ->
     gen_iq_handler:remove_iq_handler(ejabberd_sm, Host,
 				     ?NS_REGISTER).
 
+depends(_Host, _Opts) ->
+    [].
+
 stream_feature_register(Acc, Host) ->
     AF = gen_mod:get_module_opt(Host, ?MODULE, access_from,
-                                          fun(A) when is_atom(A) -> A end,
+                                          fun(A) -> A end,
 					  all),
     case (AF /= none) and lists:keymember(<<"mechanisms">>, 2, Acc) of
 	true ->
@@ -121,25 +124,25 @@ process_iq(From, To,
 		       end,
     case Type of
       set ->
-	  UTag = xml:get_subtag(SubEl, <<"username">>),
-	  PTag = xml:get_subtag(SubEl, <<"password">>),
-	  RTag = xml:get_subtag(SubEl, <<"remove">>),
+	  UTag = fxml:get_subtag(SubEl, <<"username">>),
+	  PTag = fxml:get_subtag(SubEl, <<"password">>),
+	  RTag = fxml:get_subtag(SubEl, <<"remove">>),
 	  Server = To#jid.lserver,
 	  Access = gen_mod:get_module_opt(Server, ?MODULE, access,
-                                          fun(A) when is_atom(A) -> A end,
+                                          fun(A) -> A end,
 					  all),
 	  AllowRemove = allow ==
 			  acl:match_rule(Server, Access, From),
 	  if (UTag /= false) and (RTag /= false) and
 	       AllowRemove ->
-		 User = xml:get_tag_cdata(UTag),
+		 User = fxml:get_tag_cdata(UTag),
 		 case From of
 		   #jid{user = User, lserver = Server} ->
 		       ejabberd_auth:remove_user(User, Server),
 		       IQ#iq{type = result, sub_el = []};
 		   _ ->
 		       if PTag /= false ->
-			      Password = xml:get_tag_cdata(PTag),
+			      Password = fxml:get_tag_cdata(PTag),
 			      case ejabberd_auth:remove_user(User, Server,
 							     Password)
 				  of
@@ -151,21 +154,28 @@ process_iq(From, To,
 				%% modules.  lists:foreach can
 				%% only return ok:
 				not_allowed ->
+				    Txt = <<"Removal is not allowed">>,
 				    IQ#iq{type = error,
-					  sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+					  sub_el = [SubEl,
+						    ?ERRT_NOT_ALLOWED(Lang, Txt)]};
 				not_exists ->
+				    Txt = <<"No such user">>,
 				    IQ#iq{type = error,
 					  sub_el =
-					      [SubEl, ?ERR_ITEM_NOT_FOUND]};
-				_ ->
+					      [SubEl,
+					       ?ERRT_ITEM_NOT_FOUND(Lang, Txt)]};
+				Err ->
+				    ?ERROR_MSG("failed to remove user ~s@~s: ~p",
+					       [User, Server, Err]),
 				    IQ#iq{type = error,
 					  sub_el =
 					      [SubEl,
 					       ?ERR_INTERNAL_SERVER_ERROR]}
 			      end;
 			  true ->
+			      Txt = <<"No password in this query">>,
 			      IQ#iq{type = error,
-				    sub_el = [SubEl, ?ERR_BAD_REQUEST]}
+				    sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt)]}
 		       end
 		 end;
 	     (UTag == false) and (RTag /= false) and AllowRemove ->
@@ -182,11 +192,13 @@ process_iq(From, To,
 		       ejabberd_auth:remove_user(User, Server),
 		       ignore;
 		   _ ->
-		       IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
+		       Txt = <<"The query is only allowed from local users">>,
+		       IQ#iq{type = error,
+			     sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]}
 		 end;
 	     (UTag /= false) and (PTag /= false) ->
-		 User = xml:get_tag_cdata(UTag),
-		 Password = xml:get_tag_cdata(PTag),
+		 User = fxml:get_tag_cdata(UTag),
+		 Password = fxml:get_tag_cdata(PTag),
 		 try_register_or_set_password(User, Server, Password,
 					      From, IQ, SubEl, Source, Lang,
 					      not IsCaptchaEnabled);
@@ -200,11 +212,14 @@ process_iq(From, To,
 							  SubEl, Source, Lang,
 							  true);
 			 _ ->
+			     Txt = <<"Incorrect data form">>,
 			     IQ#iq{type = error,
-				   sub_el = [SubEl, ?ERR_BAD_REQUEST]}
+				   sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt)]}
 		       end;
 		   {error, malformed} ->
-		       IQ#iq{type = error, sub_el = [SubEl, ?ERR_BAD_REQUEST]};
+		       Txt = <<"Incorrect CAPTCHA submit">>,
+		       IQ#iq{type = error,
+			     sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt)]};
 		   _ ->
 		       ErrText = <<"The CAPTCHA verification has failed">>,
 		       IQ#iq{type = error,
@@ -344,7 +359,8 @@ try_register_or_set_password(User, Server, Password,
 		      IQ#iq{type = error, sub_el = [SubEl, Error]}
 		end;
 	    deny ->
-		IQ#iq{type = error, sub_el = [SubEl, ?ERR_FORBIDDEN]}
+		Txt = <<"Denied by ACL">>,
+		IQ#iq{type = error, sub_el = [SubEl, ?ERRT_FORBIDDEN(Lang, Txt)]}
 	  end;
       _ ->
 	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
@@ -359,16 +375,24 @@ try_set_password(User, Server, Password, IQ, SubEl,
 	      of
 	    ok -> IQ#iq{type = result, sub_el = []};
 	    {error, empty_password} ->
-		IQ#iq{type = error, sub_el = [SubEl, ?ERR_BAD_REQUEST]};
+		Txt = <<"Empty password">>,
+		IQ#iq{type = error, sub_el = [SubEl, ?ERRT_BAD_REQUEST(Lang, Txt)]};
 	    {error, not_allowed} ->
-		IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+		Txt = <<"Changing password is not allowed">>,
+		IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
 	    {error, invalid_jid} ->
 		IQ#iq{type = error,
-		      sub_el = [SubEl, ?ERR_ITEM_NOT_FOUND]};
-	    _ ->
+		      sub_el = [SubEl, ?ERR_JID_MALFORMED]};
+	    Err ->
+		?ERROR_MSG("failed to register user ~s@~s: ~p",
+			   [User, Server, Err]),
 		IQ#iq{type = error,
 		      sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
 	  end;
+      error_preparing_password ->
+	  ErrText = <<"The password contains unacceptable characters">>,
+	  IQ#iq{type = error,
+		sub_el = [SubEl, ?ERRT_NOT_ACCEPTABLE(Lang, ErrText)]};
       false ->
 	  ErrText = <<"The password is too weak">>,
 	  IQ#iq{type = error,
@@ -377,18 +401,18 @@ try_set_password(User, Server, Password, IQ, SubEl,
 
 try_register(User, Server, Password, SourceRaw, Lang) ->
     case jid:is_nodename(User) of
-      false -> {error, ?ERR_BAD_REQUEST};
+      false -> {error, ?ERRT_BAD_REQUEST(Lang, <<"Malformed username">>)};
       _ ->
 	  JID = jid:make(User, Server, <<"">>),
 	  Access = gen_mod:get_module_opt(Server, ?MODULE, access,
-                                          fun(A) when is_atom(A) -> A end,
+                                          fun(A) -> A end,
 					  all),
 	  IPAccess = get_ip_access(Server),
 	  case {acl:match_rule(Server, Access, JID),
 		check_ip_access(SourceRaw, IPAccess)}
 	      of
-	    {deny, _} -> {error, ?ERR_FORBIDDEN};
-	    {_, deny} -> {error, ?ERR_FORBIDDEN};
+	    {deny, _} -> {error, ?ERRT_FORBIDDEN(Lang, <<"Denied by ACL">>)};
+	    {_, deny} -> {error, ?ERRT_FORBIDDEN(Lang, <<"Denied by ACL">>)};
 	    {allow, allow} ->
 		Source = may_remove_resource(SourceRaw),
 		case check_timeout(Source) of
@@ -406,18 +430,29 @@ try_register(User, Server, Password, SourceRaw, Lang) ->
 			      Error ->
 				  remove_timeout(Source),
 				  case Error of
-				    {atomic, exists} -> {error, ?ERR_CONFLICT};
+				    {atomic, exists} ->
+					Txt = <<"User already exists">>,
+					{error, ?ERRT_CONFLICT(Lang, Txt)};
 				    {error, invalid_jid} ->
 					{error, ?ERR_JID_MALFORMED};
 				    {error, not_allowed} ->
 					{error, ?ERR_NOT_ALLOWED};
 				    {error, too_many_users} ->
-					{error, ?ERR_NOT_ALLOWED};
-				    {error, _Reason} ->
+					Txt = <<"Too many users registered">>,
+					{error, ?ERRT_RESOURCE_CONSTRAINT(Lang, Txt)};
+				    {error, _} ->
+					?ERROR_MSG("failed to register user "
+						   "~s@~s: ~p",
+						   [User, Server, Error]),
 					{error, ?ERR_INTERNAL_SERVER_ERROR}
 				  end
 			    end;
+			error_preparing_password ->
+			    remove_timeout(Source),
+			    ErrText = <<"The password contains unacceptable characters">>,
+			    {error, ?ERRT_NOT_ACCEPTABLE(Lang, ErrText)};
 			false ->
+			    remove_timeout(Source),
 			    ErrText = <<"The password is too weak">>,
 			    {error, ?ERRT_NOT_ACCEPTABLE(Lang, ErrText)}
 		      end;
@@ -496,7 +531,7 @@ check_from(#jid{user = <<"">>, server = <<"">>},
     allow;
 check_from(JID, Server) ->
     Access = gen_mod:get_module_opt(Server, ?MODULE, access_from,
-                                    fun(A) when is_atom(A) -> A end,
+                                    fun(A) -> A end,
                                     none),
     acl:match_rule(Server, Access, JID).
 
@@ -599,7 +634,7 @@ write_time({{Y, Mo, D}, {H, Mi, S}}) ->
 		  [Y, Mo, D, H, Mi, S]).
 
 process_xdata_submit(El) ->
-    case xml:get_subtag(El, <<"x">>) of
+    case fxml:get_subtag(El, <<"x">>) of
       false -> error;
       Xdata ->
 	  Fields = jlib:parse_xdata_submit(Xdata),
@@ -612,6 +647,14 @@ process_xdata_submit(El) ->
     end.
 
 is_strong_password(Server, Password) ->
+    case jid:resourceprep(Password) of
+	PP when is_binary(PP) ->
+	    is_strong_password2(Server, Password);
+	error ->
+	    error_preparing_password
+    end.
+
+is_strong_password2(Server, Password) ->
     LServer = jid:nameprep(Server),
     case gen_mod:get_module_opt(LServer, ?MODULE, password_strength,
                                 fun(N) when is_number(N), N>=0 -> N end,
@@ -696,13 +739,13 @@ check_ip_access(IPAddress, IPAccess) ->
     acl:match_rule(global, IPAccess, IPAddress).
 
 mod_opt_type(access) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(access_from) ->
     fun (A) when is_atom(A) -> A end;
 mod_opt_type(captcha_protected) ->
     fun (B) when is_boolean(B) -> B end;
 mod_opt_type(ip_access) ->
-    fun (A) when is_atom(A) -> A end;
+    fun acl:access_rules_validator/1;
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
 mod_opt_type(password_strength) ->
     fun (N) when is_number(N), N >= 0 -> N end;
