@@ -5,7 +5,7 @@
 %%% Created :  4 May 2008 by Badlop <badlop@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -55,7 +55,7 @@
 
 -behaviour(gen_mod).
 
--export([start/2, stop/1, process/2, mod_opt_type/1]).
+-export([start/2, stop/1, process/2, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -76,6 +76,9 @@ start(_Host, _Opts) ->
 
 stop(_Host) -> ok.
 
+depends(_Host, _Opts) ->
+    [{mod_register, hard}].
+
 %%%----------------------------------------------------------------------
 %%% HTTP handlers
 %%%----------------------------------------------------------------------
@@ -113,8 +116,8 @@ process([<<"new">>],
     end;
 process([<<"delete">>],
 	#request{method = 'POST', q = Q, lang = Lang,
-		 host = Host}) ->
-    case form_del_post(Q, Host) of
+		 host = _HTTPHost}) ->
+    case form_del_post(Q) of
       {atomic, ok} ->
 	  Text = (?T(<<"Your Jabber account was successfully "
 		       "deleted.">>)),
@@ -129,8 +132,8 @@ process([<<"delete">>],
 %% should include the host where the POST was sent.
 process([<<"change_password">>],
 	#request{method = 'POST', q = Q, lang = Lang,
-		 host = Host}) ->
-    case form_changepass_post(Q, Host) of
+		 host = _HTTPHost}) ->
+    case form_changepass_post(Q) of
       {atomic, ok} ->
 	  Text = (?T(<<"The password of your Jabber account "
 		       "was successfully changed.">>)),
@@ -282,9 +285,9 @@ form_new_post(Q) ->
     case catch get_register_parameters(Q) of
       [Username, Host, Password, Password, Id, Key] ->
 	  form_new_post(Username, Host, Password, {Id, Key});
-      [_Username, _Password, _Password2, false, false] ->
+      [_Username, _Host, _Password, _Password2, false, false] ->
 	  {error, passwords_not_identical};
-      [_Username, _Password, _Password2, Id, Key] ->
+      [_Username, _Host, _Password, _Password2, Id, Key] ->
 	  ejabberd_captcha:check_captcha(Id, Key),
 	  {error, passwords_not_identical};
       _ -> {error, wrong_parameters}
@@ -361,7 +364,8 @@ form_changepass_get(Host, Lang) ->
 			   ?INPUTS(<<"text">>, <<"username">>, <<"">>,
 				   <<"20">>)]),
 		      ?XE(<<"li">>,
-			  [?CT(<<"Server:">>), ?C(<<" ">>), ?C(Host)]),
+			  [?CT(<<"Server:">>), ?C(<<" ">>),
+			   ?INPUTS(<<"text">>, <<"host">>, Host, <<"20">>)]),
 		      ?XE(<<"li">>,
 			  [?CT(<<"Old Password:">>), ?C(<<" ">>),
 			   ?INPUTS(<<"password">>, <<"passwordold">>, <<"">>,
@@ -386,12 +390,12 @@ form_changepass_get(Host, Lang) ->
 %%% Formulary change password POST
 %%%----------------------------------------------------------------------
 
-form_changepass_post(Q, Host) ->
+form_changepass_post(Q) ->
     case catch get_changepass_parameters(Q) of
-      [Username, PasswordOld, Password, Password] ->
+      [Username, Host, PasswordOld, Password, Password] ->
 	  try_change_password(Username, Host, PasswordOld,
 			      Password);
-      [_Username, _PasswordOld, _Password, _Password2] ->
+      [_Username, _Host, _PasswordOld, _Password, _Password2] ->
 	  {error, passwords_not_identical};
       _ -> {error, wrong_parameters}
     end.
@@ -405,7 +409,7 @@ get_changepass_parameters(Q) ->
 		      {value, {_Key, Value}} = lists:keysearch(Key, 1, Q),
 		      Value
 	      end,
-	      [<<"username">>, <<"passwordold">>, <<"password">>,
+	      [<<"username">>, <<"host">>, <<"passwordold">>, <<"password">>,
 	       <<"password2">>]).
 
 try_change_password(Username, Host, PasswordOld,
@@ -437,7 +441,7 @@ check_account_exists(Username, Host) ->
     end.
 
 check_password(Username, Host, Password) ->
-    case ejabberd_auth:check_password(Username, Host,
+    case ejabberd_auth:check_password(Username, <<"">>, Host,
 				      Password)
 	of
       true -> password_correct;
@@ -470,7 +474,8 @@ form_del_get(Host, Lang) ->
 			   ?INPUTS(<<"text">>, <<"username">>, <<"">>,
 				   <<"20">>)]),
 		      ?XE(<<"li">>,
-			  [?CT(<<"Server:">>), ?C(<<" ">>), ?C(Host)]),
+			  [?CT(<<"Server:">>), ?C(<<" ">>),
+			   ?INPUTS(<<"text">>, <<"host">>, Host, <<"20">>)]),
 		      ?XE(<<"li">>,
 			  [?CT(<<"Password:">>), ?C(<<" ">>),
 			   ?INPUTS(<<"password">>, <<"password">>, <<"">>,
@@ -489,7 +494,7 @@ form_del_get(Host, Lang) ->
 %%                                    {error, invalid_jid}
 register_account(Username, Host, Password) ->
     Access = gen_mod:get_module_opt(Host, mod_register, access,
-                                    fun(A) when is_atom(A) -> A end,
+                                    fun(A) -> A end,
                                     all),
     case jid:make(Username, Host, <<"">>) of
       error -> {error, invalid_jid};
@@ -513,9 +518,9 @@ register_account2(Username, Host, Password) ->
 %%% Formulary delete POST
 %%%----------------------------------------------------------------------
 
-form_del_post(Q, Host) ->
+form_del_post(Q) ->
     case catch get_unregister_parameters(Q) of
-      [Username, Password] ->
+      [Username, Host, Password] ->
 	  try_unregister_account(Username, Host, Password);
       _ -> {error, wrong_parameters}
     end.
@@ -529,7 +534,7 @@ get_unregister_parameters(Q) ->
 		      {value, {_Key, Value}} = lists:keysearch(Key, 1, Q),
 		      Value
 	      end,
-	      [<<"username">>, <<"password">>]).
+	      [<<"username">>, <<"host">>, <<"password">>]).
 
 try_unregister_account(Username, Host, Password) ->
     try unregister_account(Username, Host, Password) of
