@@ -7,7 +7,7 @@
 %%% Created :  5 Mar 2005 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -41,22 +41,19 @@
 
 -export([get_user_roster/2, get_subscription_lists/3,
 	 get_jid_info/4, process_item/2, in_subscription/6,
-	 out_subscription/4, mod_opt_type/1, opt_type/1]).
+	 out_subscription/4, mod_opt_type/1, opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
 -include("jlib.hrl").
 -include("mod_roster.hrl").
-
 -include("eldap.hrl").
 
 -define(CACHE_SIZE, 1000).
-
--define(USER_CACHE_VALIDITY, 300).
-
+-define(USER_CACHE_VALIDITY, 300).  %% in seconds
 -define(GROUP_CACHE_VALIDITY, 300).
-
--define(LDAP_SEARCH_TIMEOUT, 5).
+-define(LDAP_SEARCH_TIMEOUT, 5).    %% Timeout for LDAP search queries in seconds
+-define(INVALID_SETTING_MSG, "~s is not properly set! ~s will not function.").
 
 -record(state,
 	{host = <<"">>                                :: binary(),
@@ -108,6 +105,9 @@ stop(Host) ->
     supervisor:terminate_child(ejabberd_sup, Proc),
     supervisor:delete_child(ejabberd_sup, Proc).
 
+depends(_Host, _Opts) ->
+    [{mod_roster, hard}].
+
 %%--------------------------------------------------------------------
 %% Hooks
 %%--------------------------------------------------------------------
@@ -334,12 +334,11 @@ get_group_users(Host, Group) ->
     {ok, State} = eldap_utils:get_state(Host, ?MODULE),
     case cache_tab:dirty_lookup(shared_roster_ldap_group,
 				{Group, Host},
-				fun () -> search_group_info(State, Group) end)
-	of
-      {ok, #group_info{members = Members}}
+				fun () -> search_group_info(State, Group) end) of
+        {ok, #group_info{members = Members}}
 	  when Members /= undefined ->
-	  Members;
-      _ -> []
+            Members;
+        _ -> []
     end.
 
 get_group_name(Host, Group) ->
@@ -381,79 +380,49 @@ search_group_info(State, Group) ->
 		    true -> fun ejabberd_auth:is_user_exists/2;
 		    _ -> fun (_U, _S) -> true end
 		  end,
-    Host = State#state.host,
     case eldap_search(State,
 		      [eldap_filter:do_sub(State#state.gfilter,
 					   [{<<"%g">>, Group}])],
 		      [State#state.group_attr, State#state.group_desc,
 		       State#state.uid])
 	of
-      [] -> error;
+        [] ->
+            error;
       LDAPEntries ->
-	  {GroupDesc, MembersLists} = lists:foldl(fun
-						    (#eldap_entry{attributes =
-								      Attrs},
-						     {DescAcc, JIDsAcc}) ->
-							case
-							  {eldap_utils:get_ldap_attr(State#state.group_attr,
-										     Attrs),
-							   eldap_utils:get_ldap_attr(State#state.group_desc,
-										     Attrs),
-							   lists:keysearch(State#state.uid,
-									   1,
-									   Attrs)}
-							    of
-							  {ID, Desc,
-							   {value,
-							    {GroupMemberAttr,
-							     Members}}}
-							      when ID /= <<"">>,
-								   GroupMemberAttr
-								     ==
-								     State#state.uid ->
-							      JIDs =
-								  lists:foldl(fun
-										({ok,
-										  UID},
-										 L) ->
-										    PUID =
-											jid:nodeprep(UID),
-										    case
-										      PUID
-											of
-										      error ->
-											  L;
-										      _ ->
-											  case
-											    AuthChecker(PUID,
-													Host)
-											      of
-											    true ->
-												[{PUID,
-												  Host}
-												 | L];
-											    _ ->
-												L
-											  end
-										    end;
-										(_,
-										 L) ->
-										    L
-									      end,
-									      [],
-									      lists:map(Extractor,
-											Members)),
-							      {Desc,
-							       [JIDs
-								| JIDsAcc]};
-							  _ ->
-							      {DescAcc, JIDsAcc}
-							end
-						  end,
-						  {Group, []}, LDAPEntries),
-	  {ok,
-	   #group_info{desc = GroupDesc,
-		       members = lists:usort(lists:flatten(MembersLists))}}
+          {GroupDesc, MembersLists} = lists:foldl(fun(Entry, Acc) ->
+                                                           extract_members(State, Extractor, AuthChecker, Entry, Acc)
+                                                   end,
+                                                   {Group, []}, LDAPEntries),
+	  {ok, #group_info{desc = GroupDesc, members = lists:usort(lists:flatten(MembersLists))}}
+    end.
+
+extract_members(State, Extractor, AuthChecker, #eldap_entry{attributes = Attrs}, {DescAcc, JIDsAcc}) ->
+    Host = State#state.host,
+    case {eldap_utils:get_ldap_attr(State#state.group_attr, Attrs),
+          eldap_utils:get_ldap_attr(State#state.group_desc, Attrs),
+          lists:keysearch(State#state.uid, 1, Attrs)} of
+        {ID, Desc, {value, {GroupMemberAttr, Members}}} when ID /= <<"">>,
+                                                             GroupMemberAttr == State#state.uid ->
+            JIDs = lists:foldl(fun({ok, UID}, L) ->
+                                       PUID = jid:nodeprep(UID),
+                                       case PUID of
+                                           error ->
+                                               L;
+                                           _ ->
+                                               case AuthChecker(PUID, Host) of
+                                                   true ->
+                                                       [{PUID, Host} | L];
+                                                   _ ->
+                                                       L
+                                               end
+                                       end;
+                                  (_, L) -> L
+                               end,
+                               [],
+                               lists:map(Extractor, Members)),
+            {Desc, [JIDs | JIDsAcc]};
+        _ ->
+            {DescAcc, JIDsAcc}
     end.
 
 search_user_name(State, User) ->
