@@ -5,7 +5,7 @@
 %%% Created : 6 Mar 2010 by Karim Gemayel <karim.gemayel@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@
 -behaviour(gen_mod).
 
 -export([start/2, stop/1, process_local_iq/3,
-	 process_sm_iq/3, mod_opt_type/1]).
+	 process_sm_iq/3, mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -55,13 +55,17 @@ stop(Host) ->
     gen_iq_handler:remove_iq_handler(ejabberd_sm, Host,
 				     ?NS_SIC).
 
+depends(_Host, _Opts) ->
+    [].
+
 process_local_iq(#jid{user = User, server = Server,
 		      resource = Resource},
 		 _To, #iq{type = get, sub_el = _SubEl} = IQ) ->
     get_ip({User, Server, Resource}, IQ);
 process_local_iq(_From, _To,
-		 #iq{type = set, sub_el = SubEl} = IQ) ->
-    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]}.
+		 #iq{type = set, sub_el = SubEl, lang = Lang} = IQ) ->
+    Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+    IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]}.
 
 process_sm_iq(#jid{user = User, server = Server,
 		   resource = Resource},
@@ -69,14 +73,17 @@ process_sm_iq(#jid{user = User, server = Server,
 	      #iq{type = get, sub_el = _SubEl} = IQ) ->
     get_ip({User, Server, Resource}, IQ);
 process_sm_iq(_From, _To,
-	      #iq{type = get, sub_el = SubEl} = IQ) ->
-    IQ#iq{type = error, sub_el = [SubEl, ?ERR_FORBIDDEN]};
+	      #iq{type = get, sub_el = SubEl, lang = Lang} = IQ) ->
+    Txt = <<"Query to another users is forbidden">>,
+    IQ#iq{type = error, sub_el = [SubEl, ?ERRT_FORBIDDEN(Lang, Txt)]};
 process_sm_iq(_From, _To,
-	      #iq{type = set, sub_el = SubEl} = IQ) ->
-    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]}.
+	      #iq{type = set, sub_el = SubEl, lang = Lang} = IQ) ->
+    Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+    IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]}.
 
 get_ip({User, Server, Resource},
-       #iq{sub_el =
+       #iq{lang = Lang,
+	   sub_el =
 	       #xmlel{name = Name, attrs = Attrs} = SubEl} =
 	   IQ) ->
     case ejabberd_sm:get_user_ip(User, Server, Resource) of
@@ -88,8 +95,9 @@ get_ip({User, Server, Resource},
 				[{xmlcdata,
 				  iolist_to_binary(jlib:ip_to_list(IP))}]}]};
       _ ->
+	  Txt = <<"User session not found">>,
 	  IQ#iq{type = error,
-		sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
+		sub_el = [SubEl, ?ERRT_INTERNAL_SERVER_ERROR(Lang, Txt)]}
     end.
 
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
