@@ -5,7 +5,7 @@
 %%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -21,7 +21,7 @@
 %%% with this program; if not, write to the Free Software Foundation, Inc.,
 %%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
-%%%----------------------------------------------------------------------
+%%%---------------------u-------------------------------------------------
 
 -module(mod_vcard_ldap).
 
@@ -40,7 +40,7 @@
 -export([start/2, start_link/2, stop/1,
 	 get_sm_features/5, process_local_iq/3, process_sm_iq/3,
 	 remove_user/1, route/4, transform_module_options/1,
-	 mod_opt_type/1, opt_type/1]).
+	 mod_opt_type/1, opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -138,6 +138,9 @@ stop(Host) ->
     supervisor:terminate_child(ejabberd_sup, Proc),
     supervisor:delete_child(ejabberd_sup, Proc).
 
+depends(_Host, _Opts) ->
+    [].
+
 terminate(_Reason, State) ->
     Host = State#state.serverhost,
     gen_iq_handler:remove_iq_handler(ejabberd_local, Host,
@@ -173,7 +176,7 @@ init([Host, Opts]) ->
 			  State#state.password, State#state.tls_options),
     case State#state.search of
       true ->
-	  ejabberd_router:register_route(State#state.myhost);
+	  ejabberd_router:register_route(State#state.myhost, Host);
       _ -> ok
     end,
     {ok, State}.
@@ -206,7 +209,8 @@ process_local_iq(_From, _To,
 		 #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  IQ#iq{type = result,
 		sub_el =
@@ -223,7 +227,7 @@ process_local_iq(_From, _To,
 					    [{xmlcdata,
 					      <<(translate:translate(Lang,
 								     <<"Erlang Jabber Server">>))/binary,
-						"\nCopyright (c) 2002-2015 ProcessOne">>}]},
+						"\nCopyright (c) 2002-2016 ProcessOne">>}]},
 				 #xmlel{name = <<"BDAY">>, attrs = [],
 					children =
 					    [{xmlcdata, <<"2002-11-16">>}]}]}]}
@@ -240,10 +244,11 @@ process_sm_iq(_From, #jid{lserver = LServer} = To,
 
 process_vcard_ldap(To, IQ, Server) ->
     {ok, State} = eldap_utils:get_state(Server, ?PROCNAME),
-    #iq{type = Type, sub_el = SubEl} = IQ,
+    #iq{type = Type, sub_el = SubEl, lang = Lang} = IQ,
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  #jid{luser = LUser} = To,
 	  LServer = State#state.serverhost,
@@ -455,15 +460,17 @@ route(State, From, To, Packet) ->
 		       XDataEl = find_xdata_el(SubEl),
 		       case XDataEl of
 			 false ->
-			     Err = jlib:make_error_reply(Packet,
-							 ?ERR_BAD_REQUEST),
+			     Txt = <<"Data form not found">>,
+			     Err = jlib:make_error_reply(
+				     Packet, ?ERRT_BAD_REQUEST(Lang, Txt)),
 			     ejabberd_router:route(To, From, Err);
 			 _ ->
 			     XData = jlib:parse_xdata_submit(XDataEl),
 			     case XData of
 			       invalid ->
-				   Err = jlib:make_error_reply(Packet,
-							       ?ERR_BAD_REQUEST),
+				   Txt = <<"Incorrect data form">>,
+				   Err = jlib:make_error_reply(
+					   Packet, ?ERRT_BAD_REQUEST(Lang, Txt)),
 				   ejabberd_router:route(To, From, Err);
 			       _ ->
 				   ResIQ = IQ#iq{type = result,
@@ -505,7 +512,8 @@ route(State, From, To, Packet) ->
 	     #iq{type = Type, xmlns = ?NS_DISCO_INFO, lang = Lang} ->
 		 case Type of
 		   set ->
-		       Err = jlib:make_error_reply(Packet, ?ERR_NOT_ALLOWED),
+		       Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+		       Err = jlib:make_error_reply(Packet, ?ERRT_NOT_ALLOWED(Lang, Txt)),
 		       ejabberd_router:route(To, From, Err);
 		   get ->
 		       Info = ejabberd_hooks:run_fold(disco_info, ServerHost,
@@ -545,10 +553,11 @@ route(State, From, To, Packet) ->
 						       ++ Info}]},
 		       ejabberd_router:route(To, From, jlib:iq_to_xml(ResIQ))
 		 end;
-	     #iq{type = Type, xmlns = ?NS_DISCO_ITEMS} ->
+	     #iq{type = Type, lang = Lang, xmlns = ?NS_DISCO_ITEMS} ->
 		 case Type of
 		   set ->
-		       Err = jlib:make_error_reply(Packet, ?ERR_NOT_ALLOWED),
+		       Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+		       Err = jlib:make_error_reply(Packet, ?ERRT_NOT_ALLOWED(Lang, Txt)),
 		       ejabberd_router:route(To, From, Err);
 		   get ->
 		       ResIQ = IQ#iq{type = result,
@@ -584,7 +593,7 @@ iq_get_vcard(Lang) ->
 		[{xmlcdata,
 		  <<(translate:translate(Lang,
 					 <<"ejabberd vCard module">>))/binary,
-		    "\nCopyright (c) 2003-2015 ProcessOne">>}]}].
+		    "\nCopyright (c) 2003-2016 ProcessOne">>}]}].
 
 -define(LFIELD(Label, Var),
 	#xmlel{name = <<"field">>,
@@ -723,7 +732,7 @@ find_xdata_el1([]) -> false;
 find_xdata_el1([#xmlel{name = Name, attrs = Attrs,
 		       children = SubEls}
 		| Els]) ->
-    case xml:get_attr_s(<<"xmlns">>, Attrs) of
+    case fxml:get_attr_s(<<"xmlns">>, Attrs) of
       ?NS_XDATA ->
 	  #xmlel{name = Name, attrs = Attrs, children = SubEls};
       _ -> find_xdata_el1(Els)
