@@ -5,7 +5,7 @@
 %%% Created : 18 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@
 -behaviour(gen_mod).
 
 -export([start/2, stop/1, process_local_iq/3,
-	 mod_opt_type/1]).
+	 mod_opt_type/1, depends/2]).
 
 -include("ejabberd.hrl").
 -include("logger.hrl").
@@ -52,11 +52,12 @@ stop(Host) ->
 
 process_local_iq(_From, To,
 		 #iq{id = _ID, type = Type, xmlns = _XMLNS,
-		     sub_el = SubEl} =
+		     sub_el = SubEl, lang = Lang} =
 		     IQ) ->
     case Type of
       set ->
-	  IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	  Txt = <<"Value 'set' of 'type' attribute is not allowed">>,
+	  IQ#iq{type = error, sub_el = [SubEl, ?ERRT_NOT_ALLOWED(Lang, Txt)]};
       get ->
 	  Host = To#jid.lserver,
 	  OS = case gen_mod:get_module_opt(Host, ?MODULE, show_os,
@@ -92,6 +93,9 @@ get_os() ->
     #xmlel{name = <<"os">>, attrs = [],
 	   children = [{xmlcdata, OS}]}.
 
+depends(_Host, _Opts) ->
+    [].
+
 mod_opt_type(iqdisc) -> fun gen_iq_handler:check_type/1;
 mod_opt_type(show_os) ->
     fun (B) when is_boolean(B) -> B end;
