@@ -5,7 +5,7 @@
 %%% Created :  1 Dec 2007 by Christophe Romain <christophe.romain@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 -export([init/3, terminate/2, options/0, features/0,
     create_node_permission/6, create_node/2, delete_node/1,
     purge_node/2, subscribe_node/8, unsubscribe_node/4,
-    publish_item/6, delete_item/4, remove_extra_items/3,
+    publish_item/7, delete_item/4, remove_extra_items/3,
     get_entity_affiliations/2, get_node_affiliations/1,
     get_affiliation/2, set_affiliation/3,
     get_entity_subscriptions/2, get_node_subscriptions/1,
@@ -71,7 +71,8 @@ options() ->
 	{max_payload_size, ?MAX_PAYLOAD_SIZE},
 	{send_last_published_item, never},
 	{deliver_notifications, true},
-	{presence_based_delivery, false}].
+	{presence_based_delivery, false},
+	{itemreply, none}].
 
 features() ->
     [<<"create-nodes">>,
@@ -98,13 +99,14 @@ subscribe_node(_Nidx, _Sender, _Subscriber, _AccessModel, _SendLast, _PresenceSu
 unsubscribe_node(_Nidx, _Sender, _Subscriber, _SubId) ->
     {error, ?ERR_FORBIDDEN}.
 
-publish_item(Nidx, Publisher, PublishModel, MaxItems, ItemId, Payload) ->
+publish_item(Nidx, Publisher, PublishModel, MaxItems, ItemId, Payload,
+	     PubOpts) ->
     case nodetree_tree:get_node(Nidx) of
 	#pubsub_node{nodeid = {Host, Node}} ->
 	    lists:foreach(fun (SubNode) ->
 			node_hometree:publish_item(SubNode#pubsub_node.id,
 			    Publisher, PublishModel, MaxItems,
-			    ItemId, Payload)
+			    ItemId, Payload, PubOpts)
 		end,
 		nodetree_tree:get_subnodes(Host, Node, Publisher)),
 	    {result, {default, broadcast, []}};
