@@ -5,7 +5,7 @@
 %%% Created : 15 Jun 2009 by Brian Cully <bjc@kublai.com>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -69,13 +69,13 @@ create_node(Key, Node, Type, Owner, Options, Parents) ->
 		Other -> Other
 	    end;
 	_ ->
-	    {error, ?ERR_CONFLICT}
+	    {error, ?ERRT_CONFLICT(?MYLANG, <<"Node already exists">>)}
     end.
 
 delete_node(Key, Node) ->
     case find_node(Key, Node) of
 	false ->
-	    {error, ?ERR_ITEM_NOT_FOUND};
+	    {error, ?ERRT_ITEM_NOT_FOUND(?MYLANG, <<"Node not found">>)};
 	Record ->
 	    lists:foreach(fun (#pubsub_node{options = Opts} = Child) ->
 			NewOpts = remove_config_parent(Node, Opts),
@@ -99,7 +99,7 @@ get_node(Host, Node, _From) ->
 
 get_node(Host, Node) ->
     case find_node(Host, Node) of
-	false -> {error, ?ERR_ITEM_NOT_FOUND};
+	false -> {error, ?ERRT_ITEM_NOT_FOUND(?MYLANG, <<"Node not found">>)};
 	Record -> Record
     end.
 
@@ -115,7 +115,7 @@ get_nodes(Key) ->
 get_parentnodes(Host, Node, _From) ->
     case find_node(Host, Node) of
 	false ->
-	    {error, ?ERR_ITEM_NOT_FOUND};
+	    {error, ?ERRT_ITEM_NOT_FOUND(?MYLANG, <<"Node not found">>)};
 	#pubsub_node{parents = Parents} ->
 	    Q = qlc:q([N
 			|| #pubsub_node{nodeid = {NHost, NNode}} = N
@@ -139,7 +139,7 @@ get_subnodes(Host, <<>>) ->
     get_subnodes_helper(Host, <<>>);
 get_subnodes(Host, Node) ->
     case find_node(Host, Node) of
-	false -> {error, ?ERR_ITEM_NOT_FOUND};
+	false -> {error, ?ERRT_ITEM_NOT_FOUND(?MYLANG, <<"Node not found">>)};
 	_ -> get_subnodes_helper(Host, Node)
     end.
 
@@ -167,12 +167,8 @@ oid(Key, Name) -> {Key, Name}.
 
 %% Key    = jlib:jid() | host()
 %% Node = string()
--spec(find_node/2 ::
-    (
-	Key :: mod_pubsub:hostPubsub(),
-	Node :: mod_pubsub:nodeId())
-    -> mod_pubsub:pubsubNode() | false
-    ).
+-spec find_node(Key :: mod_pubsub:hostPubsub(), Node :: mod_pubsub:nodeId()) ->
+		       mod_pubsub:pubsubNode() | false.
 find_node(Key, Node) ->
     case mnesia:read(pubsub_node, oid(Key, Node), read) of
 	[] -> false;
@@ -188,14 +184,11 @@ find_opt(Key, Default, Options) ->
 	_ -> Default
     end.
 
--spec(traversal_helper/4 ::
-    (
-	Pred    :: fun(),
-		    Tr      :: fun(),
-				Host    :: mod_pubsub:hostPubsub(),
-				Nodes :: [mod_pubsub:nodeId(),...])
-				-> [{Depth::non_neg_integer(), Nodes::[mod_pubsub:pubsubNode(),...]}]
-				).
+-spec traversal_helper(Pred :: fun(), Tr :: fun(), Host :: mod_pubsub:hostPubsub(),
+		       Nodes :: [mod_pubsub:nodeId(),...]) ->
+			      [{Depth::non_neg_integer(),
+				Nodes::[mod_pubsub:pubsubNode(),...]}].
+
 traversal_helper(Pred, Tr, Host, Nodes) ->
     traversal_helper(Pred, Tr, 0, Host, Nodes, []).
 
@@ -220,15 +213,10 @@ remove_config_parent(Node, [{collection, Parents} | T], Acc) ->
 remove_config_parent(Node, [H | T], Acc) ->
     remove_config_parent(Node, T, [H | Acc]).
 
--spec(validate_parentage/3 ::
-    (
-	Key            :: mod_pubsub:hostPubsub(),
-	Owners         :: [ljid(),...],
-	Parent_Nodes :: [mod_pubsub:nodeId()])
-    -> true
-    %%%
-    | {error, xmlel()}
-    ).
+-spec validate_parentage(Key :: mod_pubsub:hostPubsub(), Owners :: [ljid(),...],
+			 Parent_Nodes :: [mod_pubsub:nodeId()]) ->
+				true | {error, xmlel()}.
+
 validate_parentage(_Key, _Owners, []) ->
     true;
 validate_parentage(Key, Owners, [[] | T]) ->
@@ -238,7 +226,7 @@ validate_parentage(Key, Owners, [<<>> | T]) ->
 validate_parentage(Key, Owners, [ParentID | T]) ->
     case find_node(Key, ParentID) of
 	false ->
-	    {error, ?ERR_ITEM_NOT_FOUND};
+	    {error, ?ERRT_ITEM_NOT_FOUND(?MYLANG, <<"Node not found">>)};
 	#pubsub_node{owners = POwners, options = POptions} ->
 	    NodeType = find_opt(node_type, ?DEFAULT_NODETYPE, POptions),
 	    MutualOwners = [O || O <- Owners, PO <- POwners, O == PO],
