@@ -5,7 +5,7 @@
 %%% Created : 29 May 2009 by Brian Cully <bjc@kublai.com>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2015   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2016   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -126,7 +126,7 @@ get_options_xform(Lang, Options) ->
 	    ++ XFields}}.
 
 parse_options_xform(XFields) ->
-    case xml:remove_cdata(XFields) of
+    case fxml:remove_cdata(XFields) of
 	[#xmlel{name = <<"x">>} = XEl] ->
 	    case jlib:parse_xdata_submit(XEl) of
 		XData when is_list(XData) ->
@@ -152,13 +152,9 @@ create_table() ->
 	Other -> Other
     end.
 
--spec(add_subscription/3 ::
-    (
-	_JID    :: ljid(),
-	_NodeId :: mod_pubsub:nodeIdx(),
-	Options :: [] | mod_pubsub:subOptions())
-    -> SubId :: mod_pubsub:subId()
-    ).
+-spec add_subscription(_JID :: ljid(), _NodeId :: mod_pubsub:nodeIdx(),
+		       Options :: [] | mod_pubsub:subOptions()) ->
+			      SubId :: mod_pubsub:subId().
 
 add_subscription(_JID, _NodeId, []) -> make_subid();
 add_subscription(_JID, _NodeId, Options) ->
@@ -166,25 +162,13 @@ add_subscription(_JID, _NodeId, Options) ->
     mnesia:write(#pubsub_subscription{subid = SubID, options = Options}),
     SubID.
 
--spec(delete_subscription/3 ::
-    (
-	_JID    :: _,
-	_NodeId :: _,
-	SubId   :: mod_pubsub:subId())
-    -> ok
-    ).
+-spec delete_subscription(_JID :: _, _NodeId :: _, SubId :: mod_pubsub:subId()) -> ok.
 
 delete_subscription(_JID, _NodeId, SubID) ->
     mnesia:delete({pubsub_subscription, SubID}).
 
--spec(read_subscription/3 ::
-    (
-	_JID    :: ljid(),
-	_NodeId :: _,
-	SubID   :: mod_pubsub:subId())
-    -> mod_pubsub:pubsubSubscription()
-    | {error, notfound}
-    ).
+-spec read_subscription(_JID :: ljid(), _NodeId :: _, SubID :: mod_pubsub:subId()) ->
+			       mod_pubsub:pubsubSubscription() | {error, notfound}.
 
 read_subscription(_JID, _NodeId, SubID) ->
     case mnesia:read({pubsub_subscription, SubID}) of
@@ -192,19 +176,13 @@ read_subscription(_JID, _NodeId, SubID) ->
 	_ -> {error, notfound}
     end.
 
--spec(write_subscription/4 ::
-    (
-	_JID    :: ljid(),
-	_NodeId :: _,
-	SubID   :: mod_pubsub:subId(),
-	Options :: mod_pubsub:subOptions())
-    -> ok
-    ).
+-spec write_subscription(_JID :: ljid(), _NodeId :: _, SubID :: mod_pubsub:subId(),
+			 Options :: mod_pubsub:subOptions()) -> ok.
 
 write_subscription(_JID, _NodeId, SubID, Options) ->
     mnesia:write(#pubsub_subscription{subid = SubID, options = Options}).
 
--spec(make_subid/0 :: () -> SubId::mod_pubsub:subId()).
+-spec make_subid() -> SubId::mod_pubsub:subId().
 make_subid() ->
     {T1, T2, T3} = p1_time_compat:timestamp(),
     iolist_to_binary(io_lib:fwrite("~.16B~.16B~.16B", [T1, T2, T3])).
@@ -237,39 +215,43 @@ var_xfield(?PUBSUB_SUBSCRIPTION_TYPE) -> subscription_type;
 var_xfield(?PUBSUB_SUBSCRIPTION_DEPTH) -> subscription_depth;
 var_xfield(_) -> {error, badarg}.
 
-val_xfield(deliver, [Val]) -> xopt_to_bool(Val);
-val_xfield(digest, [Val]) -> xopt_to_bool(Val);
-val_xfield(digest_frequency, [Val]) ->
+val_xfield(deliver = Opt, [Val]) -> xopt_to_bool(Opt, Val);
+val_xfield(digest = Opt, [Val]) -> xopt_to_bool(Opt, Val);
+val_xfield(digest_frequency = Opt, [Val]) ->
     case catch jlib:binary_to_integer(Val) of
 	N when is_integer(N) -> N;
-	_ -> {error, ?ERR_NOT_ACCEPTABLE}
+	_ ->
+	    Txt = <<"Value of '~s' should be integer">>,
+	    ErrTxt = iolist_to_binary(io_lib:format(Txt, [Opt])),
+	    {error, ?ERRT_NOT_ACCEPTABLE(?MYLANG, ErrTxt)}
     end;
 val_xfield(expire, [Val]) -> jlib:datetime_string_to_timestamp(Val);
-val_xfield(include_body, [Val]) -> xopt_to_bool(Val);
+val_xfield(include_body = Opt, [Val]) -> xopt_to_bool(Opt, Val);
 val_xfield(show_values, Vals) -> Vals;
 val_xfield(subscription_type, [<<"items">>]) -> items;
 val_xfield(subscription_type, [<<"nodes">>]) -> nodes;
 val_xfield(subscription_depth, [<<"all">>]) -> all;
-val_xfield(subscription_depth, [Depth]) ->
+val_xfield(subscription_depth = Opt, [Depth]) ->
     case catch jlib:binary_to_integer(Depth) of
 	N when is_integer(N) -> N;
-	_ -> {error, ?ERR_NOT_ACCEPTABLE}
+	_ ->
+	    Txt = <<"Value of '~s' should be integer">>,
+	    ErrTxt = iolist_to_binary(io_lib:format(Txt, [Opt])),
+	    {error, ?ERRT_NOT_ACCEPTABLE(?MYLANG, ErrTxt)}
     end.
 
 %% Convert XForm booleans to Erlang booleans.
-xopt_to_bool(<<"0">>) -> false;
-xopt_to_bool(<<"1">>) -> true;
-xopt_to_bool(<<"false">>) -> false;
-xopt_to_bool(<<"true">>) -> true;
-xopt_to_bool(_) -> {error, ?ERR_NOT_ACCEPTABLE}.
+xopt_to_bool(_, <<"0">>) -> false;
+xopt_to_bool(_, <<"1">>) -> true;
+xopt_to_bool(_, <<"false">>) -> false;
+xopt_to_bool(_, <<"true">>) -> true;
+xopt_to_bool(Option, _) ->
+    Txt = <<"Value of '~s' should be boolean">>,
+    ErrTxt = iolist_to_binary(io_lib:format(Txt, [Option])),
+    {error, ?ERRT_NOT_ACCEPTABLE(?MYLANG, ErrTxt)}.
 
--spec(get_option_xfield/3 ::
-    (
-	Lang :: binary(),
-	Key  :: atom(),
-	Options :: mod_pubsub:subOptions())
-    -> xmlel()
-    ).
+-spec get_option_xfield(Lang :: binary(), Key :: atom(),
+			Options :: mod_pubsub:subOptions()) -> xmlel().
 
 %% Return a field for an XForm for Key, with data filled in, if
 %% applicable, from Options.
